from typing import List, Any
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class IAMCloudElevatedMembers(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure cloud member does not have elevated access."
        id = "CKV_YC_13"
        categories = (CheckCategories.IAM,)
        supported_resources = ("yandex_resourcemanager_cloud_iam_binding", "yandex_resourcemanager_cloud_iam_member")
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "role"

    def get_forbidden_values(self) -> List[Any]:
        return ["admin", "editor"]


check = IAMCloudElevatedMembers()



from __future__ import annotations

import logging
import os
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any, cast

from checkov.arm.graph_builder.local_graph import ArmLocalGraph
from checkov.arm.graph_manager import ArmGraphManager
from checkov.arm.registry import arm_resource_registry, arm_parameter_registry
from checkov.arm.utils import get_scannable_file_paths, get_files_definitions, ARM_POSSIBLE_ENDINGS, ArmElements
from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import BaseRunner, CHECKOV_CREATE_GRAPH
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.secrets import omit_secret_value_from_checks
from checkov.runner_filter import RunnerFilter
from checkov.arm.context_parser import ContextParser

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.common.typing import LibraryGraphConnector, _CheckResult


class Runner(BaseRunner[ArmGraphManager]):
    check_type = CheckType.ARM  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.ARM,
        graph_class: type[ArmLocalGraph] = ArmLocalGraph,
        graph_manager: ArmGraphManager | None = None,
        external_registries: list[BaseRegistry] | None = None,
    ) -> None:
        super().__init__(file_extensions=ARM_POSSIBLE_ENDINGS)

        db_connector = db_connector or self.db_connector
        self.external_registries = external_registries if external_registries else []
        self.graph_class = graph_class
        self.graph_manager: "ArmGraphManager" = (
            graph_manager if graph_manager else ArmGraphManager(source=source, db_connector=db_connector)
        )
        self.graph_registry = get_graph_checks_registry(self.check_type)

        # need to check, how to support subclass differences
        self.definitions: "dict[str, dict[str, Any]]" = {}  # type:ignore[assignment]
        self.definitions_raw: "dict[str, list[tuple[int, str]]]" = {}
        self.root_folder: "str | None" = None

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        files_list: "Iterable[str]" = []
        filepath_fn = None
        if external_checks_dir:
            for directory in external_checks_dir:
                arm_resource_registry.load_external_checks(directory)

                if CHECKOV_CREATE_GRAPH and self.graph_registry:
                    self.graph_registry.load_external_checks(directory)

        if files:
            files_list = files.copy()

        if root_folder:
            filepath_fn = lambda f: f"/{os.path.relpath(f, os.path.commonprefix((root_folder, f)))}"
            self.root_folder = root_folder

            files_list = get_scannable_file_paths(root_folder=root_folder, excluded_paths=runner_filter.excluded_paths)

        self.definitions, self.definitions_raw = get_files_definitions(files_list, filepath_fn)

        if CHECKOV_CREATE_GRAPH and self.graph_registry and self.graph_manager:
            logging.info("Creating ARM graph")
            local_graph = self.graph_manager.build_graph_from_definitions(definitions=self.definitions)
            logging.info("Successfully created ARM graph")

            self.graph_manager.save_graph(local_graph)

        self.pbar.initiate(len(self.definitions))

        # run Python checks
        self.add_python_check_results(report=report, runner_filter=runner_filter, root_folder=root_folder)

        # run graph checks
        if CHECKOV_CREATE_GRAPH and self.graph_registry:
            self.add_graph_check_results(report=report, runner_filter=runner_filter)

        return report

    def add_python_check_results(self, report: Report, runner_filter: RunnerFilter, root_folder: str | None) -> None:
        """Adds Python check results to given report"""

        for arm_file in self.definitions.keys():
            self.pbar.set_additional_data({"Current File Scanned": os.path.relpath(arm_file, root_folder)})
            # There are a few cases here. If -f was used, there could be a leading / because it's an absolute path,
            # or there will be no leading slash; root_folder will always be none.
            # If -d is used, root_folder will be the value given, and -f will start with a / (hardcoded above).
            # The goal here is simply to get a valid path to the file (which arm_file does not always give).
            if arm_file[0] == "/":
                path_to_convert = (root_folder + arm_file) if root_folder else arm_file
            else:
                path_to_convert = (os.path.join(root_folder, arm_file)) if root_folder else arm_file

            file_abs_path = os.path.abspath(path_to_convert)

            if isinstance(self.definitions[arm_file], dict):
                arm_context_parser = ContextParser(arm_file, self.definitions[arm_file], self.definitions_raw[arm_file])
                logging.debug(f"Template Dump for {arm_file}: {self.definitions[arm_file]}")

                if ArmElements.RESOURCES in self.definitions[arm_file]:
                    arm_context_parser.evaluate_default_parameters()

                    # Split out nested resources from base resource
                    for resource in self.definitions[arm_file][ArmElements.RESOURCES]:
                        if isinstance(resource, dict) and "parent_name" in resource.keys():
                            continue
                        nested_resources = arm_context_parser.search_deep_keys(ArmElements.RESOURCES, resource, [])
                        if nested_resources:
                            for nr in nested_resources:
                                nr_element = nr.pop()
                                if nr_element:
                                    for element in nr_element:
                                        new_resource = element
                                        if isinstance(new_resource, dict):
                                            new_resource["parent_name"] = resource.get("name", "")
                                            new_resource["parent_type"] = resource.get("type", "")
                                            self.definitions[arm_file][ArmElements.RESOURCES].append(new_resource)

                    for resource in self.definitions[arm_file][ArmElements.RESOURCES]:
                        resource_id = arm_context_parser.extract_arm_resource_id(resource)
                        resource_name = arm_context_parser.extract_arm_resource_name(resource)
                        if resource_id is None or resource_name is None:
                            logging.debug(f"Could not determine 'resource_id' of Resource {resource}")
                            continue

                        report.add_resource(f"{arm_file}:{resource_id}")
                        entity_lines_range, entity_code_lines = arm_context_parser.extract_arm_resource_code_lines(
                            resource
                        )
                        if entity_lines_range and entity_code_lines:
                            # TODO - Variable Eval Message!
                            variable_evaluations: "dict[str, Any]" = {}

                            skipped_checks = ContextParser.collect_skip_comments(resource)

                            results = arm_resource_registry.scan(
                                arm_file,
                                {resource_name: resource},
                                skipped_checks,
                                runner_filter,
                                report_type=CheckType.ARM,
                            )

                            if results:
                                for check, check_result in results.items():
                                    record = Record(
                                        check_id=check.id,
                                        bc_check_id=check.bc_id,
                                        check_name=check.name,
                                        check_result=check_result,
                                        code_block=entity_code_lines,
                                        file_path=arm_file,
                                        file_line_range=entity_lines_range,
                                        resource=resource_id,
                                        evaluations=variable_evaluations,
                                        check_class=check.__class__.__module__,
                                        file_abs_path=file_abs_path,
                                        severity=check.severity,
                                    )
                                    record.set_guideline(check.guideline)
                                    report.add_record(record=record)
                            else:
                                # resources without checks, but not existing ones
                                report.extra_resources.add(
                                    ExtraResource(
                                        file_abs_path=file_abs_path,
                                        file_path=arm_file,
                                        resource=resource_id,
                                    )
                                )

                if ArmElements.PARAMETERS in self.definitions[arm_file]:
                    parameters = self.definitions[arm_file][ArmElements.PARAMETERS]
                    for parameter_name, parameter_details in parameters.items():
                        # TODO - Variable Eval Message!
                        variable_evaluations = {}

                        resource_id = f"parameter.{parameter_name}"
                        resource_name = cast(str, parameter_name)
                        entity_lines_range, entity_code_lines = arm_context_parser.extract_arm_resource_code_lines(
                            parameter_details
                        )

                        if entity_lines_range and entity_code_lines:
                            skipped_checks = ContextParser.collect_skip_comments(parameter_details)
                            results = arm_parameter_registry.scan(
                                arm_file, {resource_name: parameter_details}, skipped_checks, runner_filter
                            )
                            for check, check_result in results.items():
                                censored_code_lines = omit_secret_value_from_checks(
                                    check=check,
                                    check_result=check_result,
                                    entity_code_lines=entity_code_lines,
                                    entity_config=parameter_details,
                                    resource_attributes_to_omit=runner_filter.resource_attr_to_omit,
                                )
                                self.build_record(
                                    report=report,
                                    check=check,
                                    check_result=check_result,
                                    code_block=censored_code_lines,
                                    file_path=arm_file,
                                    file_abs_path=file_abs_path,
                                    file_line_range=entity_lines_range,
                                    resource_id=resource_id,
                                    evaluations=variable_evaluations,
                                )

            self.pbar.update()
        self.pbar.close()

    def add_graph_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        """Adds graph check results to given report"""

        graph_checks_results = self.run_graph_checks_results(runner_filter, self.check_type)

        for check, check_results in graph_checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path: str = entity[CustomAttributes.FILE_PATH]
                start_line = entity[START_LINE] - 1
                end_line = entity[END_LINE] - 1

                self.build_record(
                    report=report,
                    check=check,
                    check_result=check_result,
                    code_block=self.definitions_raw[entity_file_path][start_line:end_line],
                    file_path=entity_file_path,
                    file_abs_path=os.path.abspath(entity_file_path),
                    file_line_range=[start_line - 1, end_line - 1],
                    resource_id=entity[CustomAttributes.ID],
                )

    def build_record(
        self,
        report: Report,
        check: BaseCheck | BaseGraphCheck,
        check_result: _CheckResult,
        code_block: list[tuple[int, str]],
        file_path: str,
        file_abs_path: str,
        file_line_range: list[int],
        resource_id: str,
        evaluations: dict[str, Any] | None = None,
    ) -> None:
        record = Record(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result=check_result,
            code_block=code_block,
            file_path=file_path,
            file_line_range=file_line_range,
            resource=resource_id,
            evaluations=evaluations,
            check_class=check.__class__.__module__,
            file_abs_path=file_abs_path,
            severity=check.severity,
        )
        record.set_guideline(check.guideline)
        report.add_record(record=record)



from __future__ import annotations

import json
import logging
import os
from collections.abc import Sequence
from typing import Any, TYPE_CHECKING, TypeVar, cast, Tuple

from lark import Tree
import re

from checkov.common.typing import TFDefinitionKeyType
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.json_utils import CustomJSONEncoder, object_hook
from checkov.terraform.modules.module_objects import TFDefinitionKey
from checkov.terraform.checks.utils.dependency_path_handler import unify_dependency_path
from checkov.terraform.graph_builder.utils import remove_module_dependency_in_path
from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_PREFIX, TERRAFORM_NESTED_MODULE_PATH_ENDING

if TYPE_CHECKING:
    from typing_extensions import TypeAlias

_Conf = TypeVar("_Conf", bound="dict[Any, Any]")

ENTITY_NAME_PATTERN = re.compile(r"[^\W0-9][\w-]*")
RESOLVED_MODULE_PATTERN = re.compile(r"\[.+\#.+\]")
_Hcl2Payload: TypeAlias = "dict[str, list[dict[str, Any]]]"
external_modules_download_path = os.environ.get('EXTERNAL_MODULES_DIR', DEFAULT_EXTERNAL_MODULES_DIR)


def is_valid_block(block: Any) -> bool:
    if not isinstance(block, dict):
        return True

    # if the block is empty, there's no need to process it further
    if not block:
        return False

    entity_name = next(iter(block.keys()))
    if re.fullmatch(ENTITY_NAME_PATTERN, entity_name):
        return True
    return False


def validate_malformed_definitions(raw_data: _Hcl2Payload) -> _Hcl2Payload:
    return {
        block_type: [block for block in blocks if is_valid_block(block)]
        for block_type, blocks in raw_data.items()
    }


def clean_bad_definitions(tf_definition_list: _Hcl2Payload) -> _Hcl2Payload:
    return {
        block_type: [
            definition
            for definition in definition_list
            if block_type in {"locals", "terraform"} or not isinstance(definition, dict) or len(definition) == 1
        ]
        for block_type, definition_list in tf_definition_list.items()
    }


def safe_index(sequence_hopefully: Sequence[Any], index: int) -> Any:
    try:
        return sequence_hopefully[index]
    except IndexError:
        logging.debug(f'Failed to parse index int ({index}) out of {sequence_hopefully}', exc_info=True)
        return None


def remove_module_dependency_from_path(path: str) -> str:
    """
    :param path: path that looks like "dir/main.tf[other_dir/x.tf#0]
    :return: only the outer path: dir/main.tf
    """
    if "#" in path:
        path = re.sub(RESOLVED_MODULE_PATTERN, '', path)
    return path


def get_module_dependency_map(
    tf_definitions: dict[str, Any]
) -> tuple[dict[str, list[list[str]]], dict[str, Any], dict[tuple[str, str], list[str]]]:
    """
    :param tf_definitions, with paths in format 'dir/main.tf[module_dir/main.tf#0]'
    :return module_dependency_map: mapping between directories and the location of its module definition:
            {'dir': 'module_dir/main.tf'}
    :return tf_definitions: with paths in format 'dir/main.tf'
    """
    module_dependency_map: dict[str, list[list[str]]] = {}
    copy_of_tf_definitions = {}
    dep_index_mapping: dict[tuple[str, str], list[str]] = {}
    origin_keys = list(filter(lambda k: not k.endswith(TERRAFORM_NESTED_MODULE_PATH_ENDING), tf_definitions.keys()))
    unevaluated_keys = list(filter(lambda k: k.endswith(TERRAFORM_NESTED_MODULE_PATH_ENDING), tf_definitions.keys()))
    for file_path in origin_keys:
        dir_name = os.path.dirname(file_path)
        module_dependency_map[dir_name] = [[]]
        copy_of_tf_definitions[file_path] = pickle_deepcopy(tf_definitions[file_path])

    next_level, unevaluated_keys = get_next_vertices(origin_keys, unevaluated_keys)
    while next_level:
        for file_path in next_level:
            path, module_dependency, module_dependency_num = remove_module_dependency_in_path(file_path)
            dir_name = os.path.dirname(path)
            current_deps = pickle_deepcopy(module_dependency_map[os.path.dirname(module_dependency)])
            for dep in current_deps:
                dep.append(module_dependency)
            if dir_name not in module_dependency_map:
                module_dependency_map[dir_name] = current_deps
            else:
                for dep in current_deps:
                    if dep not in module_dependency_map[dir_name]:
                        module_dependency_map[dir_name].append(dep)
            copy_of_tf_definitions[path] = pickle_deepcopy(tf_definitions[file_path])
            origin_keys.append(path)
            dep_index_mapping.setdefault((path, module_dependency), []).append(module_dependency_num)
        next_level, unevaluated_keys = get_next_vertices(origin_keys, unevaluated_keys)
    for key, dep_trails in module_dependency_map.items():
        hashes = set()
        deduped = []
        for trail in dep_trails:
            trail_hash = unify_dependency_path(trail)
            if trail_hash in hashes:
                continue
            hashes.add(trail_hash)
            deduped.append(trail)
        module_dependency_map[key] = deduped
    return module_dependency_map, copy_of_tf_definitions, dep_index_mapping


def get_next_vertices(evaluated_files: list[str], unevaluated_files: list[str]) -> tuple[list[str], list[str]]:
    """
    This function implements a lazy separation of levels for the evaluated files. It receives the evaluated
    files, and returns 2 lists:
    1. The next level of files - files from the unevaluated_files which have no unresolved dependency (either
        no dependency or all dependencies were evaluated).
    2. unevaluated - files which have yet to be evaluated, and still have pending dependencies

    Let's say we have this dependency tree:
    a -> b
    x -> b
    y -> c
    z -> b
    b -> c
    c -> d

    The first run will return [a, y, x, z] as the next level since all of them have no dependencies
    The second run with the evaluated being [a, y, x, z] will return [b] as the next level.
    Please mind that [c] has some resolved dependencies (from y), but has unresolved dependencies from [b].
    The third run will return [c], and the fourth will return [d].
    """

    next_level, unevaluated, do_not_eval_yet = [], [], []
    for key in unevaluated_files:
        found = False
        for eval_key in evaluated_files:
            if eval_key in key:
                found = True
                break
        if not found:
            do_not_eval_yet.append(key.split(TERRAFORM_NESTED_MODULE_PATH_PREFIX)[0])
            unevaluated.append(key)
        else:
            next_level.append(key)

    move_to_uneval = list(filter(lambda k: k.split(TERRAFORM_NESTED_MODULE_PATH_PREFIX)[0] in do_not_eval_yet, next_level))
    for k in move_to_uneval:
        next_level.remove(k)
        unevaluated.append(k)
    return next_level, unevaluated


def clean_parser_types(conf: _Conf) -> _Conf:
    if not conf:
        return conf

    sorted_keys = list(conf.keys())
    first_key_type = type(sorted_keys[0])
    if first_key_type is None:
        return {}

    if all(isinstance(x, first_key_type) for x in sorted_keys):
        sorted_keys.sort()

    # Create a new dict where the keys are sorted alphabetically
    sorted_conf = {key: conf[key] for key in sorted_keys}
    for attribute, values in sorted_conf.items():
        if attribute == 'alias':
            continue
        if isinstance(values, list):
            sorted_conf[attribute] = clean_parser_types_lst(values)
        elif isinstance(values, dict):
            sorted_conf[attribute] = clean_parser_types(values)
        elif isinstance(values, str) and values in ('true', 'false'):
            sorted_conf[attribute] = True if values == 'true' else False
        elif isinstance(values, set):
            sorted_conf[attribute] = clean_parser_types_lst(list(values))
        elif isinstance(values, Tree):
            sorted_conf[attribute] = str(values)
    return sorted_conf  # type:ignore[return-value]  # still the same type as before


def clean_parser_types_lst(values: list[Any]) -> list[Any]:
    for idx, val in enumerate(values):
        if isinstance(val, dict):
            values[idx] = clean_parser_types(val)
        elif isinstance(val, list):
            values[idx] = clean_parser_types_lst(val)
        elif isinstance(val, str):
            if val == 'true':
                values[idx] = True
            elif val == 'false':
                values[idx] = False
        elif isinstance(val, set):
            values[idx] = clean_parser_types_lst(list(val))
    str_values_in_lst = []
    result_values = []
    for val in values:
        if isinstance(val, str):
            str_values_in_lst.append(val)
        else:
            result_values.append(val)
    str_values_in_lst.sort()
    result_values.extend(str_values_in_lst)
    return result_values


def serialize_definitions(tf_definitions: _Conf) -> _Conf:
    return cast("_Conf", json.loads(json.dumps(tf_definitions, cls=CustomJSONEncoder), object_hook=object_hook))


def get_module_from_full_path(file_path: TFDefinitionKey | None) -> Tuple[TFDefinitionKey | None, None]:
    if not file_path or not is_nested(file_path):
        return None, None
    if file_path.tf_source_modules is None:
        return None, None
    return TFDefinitionKey(file_path=file_path.tf_source_modules.path, tf_source_modules=file_path.tf_source_modules.nested_tf_module), None


def get_module_name(file_path: TFDefinitionKey) -> str | None:
    if not file_path.tf_source_modules:
        return None
    module_name = file_path.tf_source_modules.name
    if file_path.tf_source_modules.foreach_idx:
        foreach_or_count = '"' if isinstance(file_path.tf_source_modules.foreach_idx, str) else ''
        module_name = f'{module_name}[{foreach_or_count}{file_path.tf_source_modules.foreach_idx}{foreach_or_count}]'
    return module_name


def is_nested(full_path: TFDefinitionKey | None) -> bool:
    return full_path.tf_source_modules is not None if full_path is not None else False


def get_abs_path(file_path: TFDefinitionKeyType) -> str:
    # file_path might be str for terraform-plan
    return file_path if isinstance(file_path, str) else str(file_path.file_path)



from __future__ import annotations

import itertools
import typing
from collections import defaultdict
from typing import Any

from checkov.common.util.consts import RESOLVED_MODULE_ENTRY_NAME
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.terraform import TFModule
from checkov.terraform.graph_builder.foreach.abstract_handler import ForeachAbstractHandler
from checkov.terraform.graph_builder.foreach.consts import FOREACH_STRING, COUNT_STRING
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock

if typing.TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph


class ForeachModuleHandler(ForeachAbstractHandler):
    def __init__(self, local_graph: TerraformLocalGraph):
        super().__init__(local_graph)

    def handle(self, modules_blocks: list[int]) -> None:
        """
        modules_blocks (list[int]): list of module blocks indexes in the graph that contains for_each / counts.
        """
        if not modules_blocks:
            return
        current_level: list[TFModule | None] = [None]
        # We use `[:]` instead of deepcopy as it's much faster and the list has only primitive types (int indexes)
        main_module_modules = self.local_graph.vertices_by_module_dependency[None][BlockType.MODULE][:]
        modules_to_render = main_module_modules

        while modules_to_render:
            modules_to_render = self._render_foreach_modules_by_levels(modules_blocks, modules_to_render, current_level)

    def _render_foreach_modules_by_levels(self, modules_blocks: list[int], modules_to_render: list[int],
                                          current_level: list[TFModule | None]) -> list[int]:
        """
        modules_blocks: The module blocks with for_each/count statement in the graph.
        modules_to_render: The list of modules indexes to render at this iteration.
        current_level: The parent current level that we are running on this iteration (first will be None).

        return: the next (list) of the modules to render.

        For example: at this folder - tests/terraform/graph/variable_rendering/resources/foreach_module_dup_foreach
        We will run over the levels by:
        First level -> s3_module and s3_module2 (Copying the module and all his dependencies)
        Second level -> inner_s3_module and inner_s3_module2 (Copying the module and all his dependencies)
        This will generate a graph with 20 modules and 16 resources.
        """
        sub_graph = self._build_sub_graph(modules_blocks)
        self._render_sub_graph(sub_graph, blocks_to_render=modules_blocks)
        for module_idx in modules_to_render:
            module_block = self.local_graph.vertices[module_idx]
            for_each = module_block.attributes.get(FOREACH_STRING)
            count = module_block.attributes.get(COUNT_STRING)
            if for_each:
                for_each = self._handle_static_statement(module_idx, sub_graph)
                if not for_each or not self._is_static_statement(module_idx, sub_graph):
                    continue
                self._duplicate_module_with_for_each(module_idx, for_each)
            elif count:
                count = self._handle_static_statement(module_idx, sub_graph)
                if not count or not self._is_static_statement(module_idx, sub_graph):
                    continue
                self._duplicate_module_with_count(module_idx, count)
        return self._get_modules_to_render(current_level)

    def _duplicate_module_with_for_each(self, module_idx: int, for_each: dict[str, Any] | list[str]) -> None:
        self._create_new_resources_foreach(for_each, module_idx)

    def _duplicate_module_with_count(self, module_idx: int, count: int) -> None:
        self._create_new_resources_count(count, module_idx)

    def _get_modules_to_render(self, current_level: list[TFModule | None]) -> list[int]:
        rendered_modules = [self.local_graph.vertices_by_module_dependency[curr][BlockType.MODULE] for curr in current_level][0]
        current_level.clear()
        for m_idx in rendered_modules:
            current_level.append(self._get_current_tf_module_object(m_idx))
        modules_to_render = [self.local_graph.vertices_by_module_dependency[curr][BlockType.MODULE] for curr in current_level]
        return list(itertools.chain.from_iterable(modules_to_render))

    def _get_current_tf_module_object(self, m_idx: int) -> TFModule:
        m = self.local_graph.vertices[m_idx]
        m_name = m.name.split('[')[0]
        return TFModule(m.path, m_name, m.source_module_object, m.for_each_index)

    def _create_new_resources_foreach(self, statement: list[str] | dict[str, Any], block_idx: int) -> None:
        # Important it will be before the super call to avoid changes occuring from super
        main_resource = self.local_graph.vertices[block_idx]
        super()._create_new_resources_foreach(statement, block_idx)

        if isinstance(statement, list):
            for i, new_value in enumerate(statement):
                should_override = True if i == 0 else False
                self._update_module_children(main_resource, new_value, should_override_foreach_key=should_override)
        elif isinstance(statement, dict):
            for i, (new_key, _) in enumerate(statement.items()):
                should_override = True if i == 0 else False
                self._update_module_children(main_resource, new_key, should_override_foreach_key=should_override)

    def _create_new_foreach_resource(self, block_idx: int, foreach_idx: int, main_resource: TerraformBlock,
                                     new_key: int | str, new_value: int | str) -> None:
        self._create_new_module(main_resource, new_value, new_key=new_key, resource_idx=block_idx,
                                foreach_idx=foreach_idx)

    def _update_module_children(self, main_resource: TerraformBlock,
                                original_foreach_or_count_key: int | str,
                                should_override_foreach_key: bool = True) -> None:
        original_module_key = TFModule(path=main_resource.path, name=main_resource.name,
                                       nested_tf_module=main_resource.source_module_object)

        if not should_override_foreach_key:
            original_module_key.foreach_idx = original_foreach_or_count_key

        self._update_children_foreach_index(original_foreach_or_count_key, original_module_key,
                                            should_override_foreach_key=should_override_foreach_key)

    def _create_new_resources_count(self, statement: int, block_idx: int) -> None:
        main_resource = self.local_graph.vertices[block_idx]
        for i in range(statement):
            self._create_new_module(main_resource, i, resource_idx=block_idx, foreach_idx=i)

        # We purposely do it at the end to avoid influencing data structures in the middle of an update
        for i in range(statement):
            should_override = True if i == 0 else False
            self._update_module_children(main_resource, i, should_override_foreach_key=should_override)

    def _update_children_foreach_index(self, original_foreach_or_count_key: int | str, original_module_key: TFModule,
                                       current_module_key: TFModule | None = None,
                                       should_override_foreach_key: bool = True) -> None:
        """
        Go through all child vertices and update source_module_object with foreach_idx
        """
        if current_module_key is None:
            current_module_key = original_module_key
        if current_module_key not in self.local_graph.vertices_by_module_dependency:
            return
        values = self.local_graph.vertices_by_module_dependency[current_module_key].values()
        for child_indexes in values:
            for child_index in child_indexes:
                child = self.local_graph.vertices[child_index]

                self._update_nested_tf_module_foreach_idx(original_foreach_or_count_key, original_module_key,
                                                          child.source_module_object)
                self._update_resolved_entry_for_tf_definition(child, original_foreach_or_count_key, original_module_key)

                # Important to copy to avoid changing the object by reference
                child_source_module_object_copy = pickle_deepcopy(child.source_module_object)
                if should_override_foreach_key and child_source_module_object_copy is not None:
                    child_source_module_object_copy.foreach_idx = None

                child_module_key = TFModule(path=child.path, name=child.name,
                                            nested_tf_module=child_source_module_object_copy,
                                            foreach_idx=child.for_each_index)
                del child_source_module_object_copy
                self._update_children_foreach_index(original_foreach_or_count_key, original_module_key,
                                                    child_module_key)

    def _create_new_module(
            self,
            main_resource: TerraformBlock,
            new_value: int | str,
            resource_idx: int,
            foreach_idx: int,
            new_key: int | str | None = None) -> None:
        new_resource = pickle_deepcopy(main_resource)
        block_name = new_resource.name
        config_attrs = new_resource.config.get(block_name, {})
        key_to_val_changes = self._build_key_to_val_changes(main_resource, new_value, new_key)
        self._update_foreach_attrs(config_attrs, key_to_val_changes, new_resource)
        idx_to_change = new_key or new_value
        new_resource.for_each_index = idx_to_change

        main_resource_module_key = TFModule(
            path=new_resource.path,
            name=main_resource.name,
            nested_tf_module=new_resource.source_module_object,
        )

        # Without making this copy the test don't pass, as we might access the data structure in the middle of an update
        copy_of_vertices_by_module_dependency = pickle_deepcopy(self.local_graph.vertices_by_module_dependency)
        main_resource_module_value = pickle_deepcopy(copy_of_vertices_by_module_dependency[main_resource_module_key])
        new_resource_module_key = TFModule(new_resource.path, new_resource.name, new_resource.source_module_object,
                                           idx_to_change)

        self._update_block_name_and_id(new_resource, idx_to_change)
        self._update_resolved_entry_for_tf_definition(new_resource, idx_to_change, main_resource_module_key)
        if foreach_idx != 0:
            self.local_graph.vertices.append(new_resource)
            self._create_new_module_with_vertices(main_resource, main_resource_module_value, resource_idx, new_resource,
                                                  new_resource_module_key)
        else:
            self.local_graph.vertices[resource_idx] = new_resource

            key_with_foreach_index = main_resource_module_key
            key_with_foreach_index.foreach_idx = idx_to_change
            self.local_graph.vertices_by_module_dependency[key_with_foreach_index] = main_resource_module_value
            self.local_graph.vertices_by_module_dependency_by_name[key_with_foreach_index][new_resource.name] = main_resource_module_value

        del copy_of_vertices_by_module_dependency, new_resource, main_resource_module_key, main_resource_module_value

    def _create_new_module_with_vertices(self, main_resource: TerraformBlock,
                                         main_resource_module_value: dict[str, list[int]],
                                         resource_idx: Any, new_resource: TerraformBlock | None = None,
                                         new_resource_module_key: TFModule | None = None) -> None:
        if new_resource is None:
            new_resource_name = main_resource.name
            new_resource_module_key = TFModule(main_resource.path, new_resource_name, main_resource.source_module_object,
                                               main_resource.for_each_index)
        else:
            new_resource_name = new_resource.name

        new_resource_vertex_idx = len(self.local_graph.vertices) - 1
        original_vertex_source_module = self.local_graph.vertices[resource_idx].source_module_object
        if original_vertex_source_module:
            source_module_key = TFModule(
                path=original_vertex_source_module.path,
                name=original_vertex_source_module.name,
                nested_tf_module=original_vertex_source_module.nested_tf_module,
            )
        else:
            source_module_key = None
        self.local_graph.vertices_by_module_dependency[source_module_key][BlockType.MODULE].append(new_resource_vertex_idx)
        self.local_graph.vertices_by_module_dependency_by_name[source_module_key][BlockType.MODULE][new_resource_name].append(new_resource_vertex_idx)
        new_vertices_module_value = self._add_new_vertices_for_module(new_resource_module_key, main_resource_module_value, new_resource_vertex_idx)
        self.local_graph.vertices_by_module_dependency.update({new_resource_module_key: new_vertices_module_value})
        self.local_graph.vertices_by_module_dependency_by_name.update({new_resource_module_key: {new_resource_name: new_vertices_module_value}})

    def _add_new_vertices_for_module(self, new_module_key: TFModule | None, new_module_value: dict[str, list[int]],
                                     new_resource_vertex_idx: int) -> dict[str, list[int]]:
        new_vertices_module_value: dict[str, list[int]] = defaultdict(list)
        for vertex_type, vertices_idx in new_module_value.items():
            for vertex_idx in vertices_idx:
                module_vertex = self.local_graph.vertices[vertex_idx]
                new_vertex = pickle_deepcopy(module_vertex)
                new_vertex.source_module_object = new_module_key
                self.local_graph.vertices.append(new_vertex)

                # Update source module based on the new added vertex
                new_vertex.source_module.pop()
                new_vertex.source_module.add(new_resource_vertex_idx)

                new_vertex_idx = len(self.local_graph.vertices) - 1
                new_vertices_module_value[vertex_type].append(new_vertex_idx)

                if vertex_type == BlockType.MODULE:
                    module_vertex_key = TFModule(path=module_vertex.path, name=module_vertex.name,
                                                 nested_tf_module=module_vertex.source_module_object,
                                                 foreach_idx=module_vertex.for_each_index)
                    module_vertex_value = self.local_graph.vertices_by_module_dependency[module_vertex_key]
                    self._create_new_module_with_vertices(new_vertex, module_vertex_value, new_vertex_idx)

        return new_vertices_module_value

    @staticmethod
    def _update_resolved_entry_for_tf_definition(child: TerraformBlock, original_foreach_or_count_key: int | str,
                                                 original_module_key: TFModule) -> None:
        if child.block_type == BlockType.RESOURCE:
            child_name, child_type = child.name.split('.')
            config = child.config[child_name][child_type]
        else:
            config = child.config.get(child.name)
        if isinstance(config, dict):
            resolved_module_name = config.get(RESOLVED_MODULE_ENTRY_NAME)
            if resolved_module_name is not None and len(resolved_module_name) > 0:
                tf_moudle: TFModule = resolved_module_name[0].tf_source_modules
                ForeachAbstractHandler._update_nested_tf_module_foreach_idx(
                    original_foreach_or_count_key,
                    original_module_key,
                    tf_moudle,
                )



from __future__ import annotations

import re
from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction

MULTI_STAGE_PATTERN = re.compile(r"(\S+)\s+as\s+(\S+)", re.IGNORECASE)


class ReferenceLatestTag(BaseDockerfileCheck):
    def __init__(self) -> None:
        name = "Ensure the base image uses a non latest version tag"
        id = "CKV_DOCKER_7"
        supported_instructions = ("FROM",)
        categories = (CheckCategories.CONVENTION,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        stages = []

        for content in conf:
            base_image = content["value"]
            if " as " in base_image.lower():
                # do an initial lookup before using the regex
                multi_stage = re.match(MULTI_STAGE_PATTERN, base_image)
                if multi_stage:
                    base_image = multi_stage[1]
                    stages.append(multi_stage[2])

            if ":" not in base_image and base_image not in stages and base_image != "scratch":
                return CheckResult.FAILED, [content]
            elif base_image.endswith(":latest"):
                return CheckResult.FAILED, [content]
        return CheckResult.PASSED, [content]


check = ReferenceLatestTag()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class EMRClusterConfEncryptsEBS(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure EMR Cluster security configuration encrypts ebs disks"
        id = "CKV_AWS_350"
        supported_resources = ("aws_emr_security_configuration",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        security_conf = conf.get("configuration")
        if security_conf and isinstance(security_conf, list) and isinstance(security_conf[0], dict):
            encrypt_conf = security_conf[0].get("EncryptionConfiguration")
            if encrypt_conf and isinstance(encrypt_conf, dict) and encrypt_conf.get("EnableAtRestEncryption") is True:
                ebs_encrypt = find_in_dict(
                    input_dict=encrypt_conf,
                    key_path="AtRestEncryptionConfiguration/LocalDiskEncryptionConfiguration/EnableEbsEncryption",
                )
                if ebs_encrypt:
                    return CheckResult.PASSED

            return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = EMRClusterConfEncryptsEBS()



from __future__ import annotations

from checkov.common.graph.graph_builder.variable_rendering.vertex_reference import VertexReference
from checkov.terraform.graph_builder.graph_components.block_types import BlockType


class TerraformVertexReference(VertexReference):
    def __init__(self, block_type: str, sub_parts: list[str], origin_value: str) -> None:
        super().__init__(block_type, sub_parts, origin_value)

    @staticmethod
    def block_type_str_to_enum(block_type_str: str) -> str:
        if block_type_str == "var":
            return BlockType.VARIABLE
        if block_type_str == "local":
            return BlockType.LOCALS
        return BlockType().get(block_type_str)



from __future__ import annotations

import logging
import subprocess  # nosec


def get_kustomize_version(kustomize_command: str) -> str | None:
    try:
        proc = subprocess.run([kustomize_command, "version"], capture_output=True)  # nosec
        version_output = proc.stdout.decode("utf-8")

        if "Version:" in version_output:
            # version <= 4 output looks like '{Version:kustomize/v4.5.7 GitCommit:...}\n'
            kustomize_version = version_output[version_output.find("/") + 1 : version_output.find("G") - 1]
        elif version_output.startswith("v"):
            # version >= 5 output looks like 'v5.0.0\n'
            kustomize_version = version_output.rstrip("\n")
        else:
            return None

        return kustomize_version
    except Exception:
        logging.debug(f"An error occured testing the {kustomize_command} command:", exc_info=True)

    return None



from __future__ import annotations

import logging
import os
import json
import itertools
from concurrent import futures
from io import StringIO
from typing import Any, TYPE_CHECKING
from collections import defaultdict

import dpath
from igraph import Graph

try:
    from networkx import DiGraph, node_link_data
except ImportError:
    logging.info("Not able to import networkx")
    DiGraph = str
    node_link_data = lambda G : {}

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.consts import SUPPORTED_FILE_EXTENSIONS
from checkov.common.typing import _ReducedScanReport
from checkov.common.util.file_utils import compress_string_io_tar
from checkov.common.util.igraph_serialization import serialize_to_json
from checkov.common.util.json_utils import CustomJSONEncoder

if TYPE_CHECKING:
    from mypy_boto3_s3.client import S3Client

    from checkov.common.output.report import Report

checkov_results_prefix = 'checkov_results'
check_reduced_keys = (
    'check_id', 'check_result', 'resource', 'file_path',
    'file_line_range', 'code_block', 'caller_file_path', 'caller_file_line_range')
secrets_check_reduced_keys = check_reduced_keys + ('validation_status',)
check_metadata_keys = ('evaluations', 'code_block', 'workflow_name', 'triggers', 'job')


def _is_scanned_file(file: str) -> bool:
    file_ending = os.path.splitext(file)[1]
    return file_ending in SUPPORTED_FILE_EXTENSIONS


def _put_json_object(s3_client: S3Client, json_obj: Any, bucket: str, object_path: str, log_stack_trace_on_error: bool = True) -> None:
    try:
        s3_client.put_object(Bucket=bucket, Key=object_path, Body=json.dumps(json_obj, cls=CustomJSONEncoder))
    except Exception:
        logging.error(f"failed to persist object into S3 bucket {bucket}", exc_info=log_stack_trace_on_error)
        raise


def _extract_checks_metadata(report: Report, full_repo_object_key: str) -> dict[str, dict[str, Any]]:
    metadata: dict[str, dict[str, Any]] = defaultdict(dict)
    for check in itertools.chain(report.passed_checks, report.failed_checks, report.skipped_checks):
        metadata_key = f'{check.file_path}:{check.resource}'
        check_meta = {k: getattr(check, k, "") for k in check_metadata_keys}
        check_meta['file_object_path'] = full_repo_object_key + check.file_path
        metadata[metadata_key][check.check_id] = check_meta

    return metadata


def reduce_scan_reports(scan_reports: list[Report]) -> dict[str, _ReducedScanReport]:
    """
    Transform checkov reports objects into compact dictionaries
    :param scan_reports: List of checkov output reports
    :return: dictionary of
    """
    reduced_scan_reports: dict[str, _ReducedScanReport] = {}
    for report in scan_reports:
        check_type = report.check_type
        reduced_keys = secrets_check_reduced_keys if check_type == CheckType.SECRETS else check_reduced_keys
        reduced_scan_reports[check_type] = \
            {
                "checks": {
                    "passed_checks": [
                        {k: getattr(check, k) for k in reduced_keys}
                        for check in report.passed_checks],
                    "failed_checks": [
                        {k: getattr(check, k) for k in reduced_keys}
                        for check in report.failed_checks],
                    "skipped_checks": [
                        {k: getattr(check, k) for k in reduced_keys}
                        for check in report.skipped_checks]
                },
                "image_cached_results": report.image_cached_results
        }
    return reduced_scan_reports


def persist_checks_results(
        reduced_scan_reports: dict[str, _ReducedScanReport], s3_client: S3Client, bucket: str,
        full_repo_object_key: str
) -> dict[str, str]:
    """
    Save reduced scan reports into bridgecrew's platform
    :return: List of checks results path of all runners
    """
    checks_results_paths = {}
    for check_type, reduced_report in reduced_scan_reports.items():
        check_result_object_path = f'{full_repo_object_key}/{checkov_results_prefix}/{check_type}/checks_results.json'
        checks_results_paths[check_type] = check_result_object_path
        _put_json_object(s3_client, reduced_report, bucket, check_result_object_path)
    return checks_results_paths


def persist_run_metadata(
        run_metadata: dict[str, str | list[str]], s3_client: S3Client, bucket: str, full_repo_object_key: str, use_checkov_results: bool = True
) -> None:
    object_path = f'{full_repo_object_key}/{checkov_results_prefix}/run_metadata.json' if use_checkov_results else f'{full_repo_object_key}/run_metadata.json'
    try:
        s3_client.put_object(Bucket=bucket, Key=object_path, Body=json.dumps(run_metadata, indent=2))

    except Exception:
        logging.error(f"failed to persist run metadata into S3 bucket {bucket}", exc_info=True)
        raise


def persist_logs_stream(logs_stream: StringIO, s3_client: S3Client, bucket: str, full_repo_object_key: str) -> None:
    file_io = compress_string_io_tar(logs_stream)
    object_path = f'{full_repo_object_key}/logs_file.tar.gz'
    try:
        s3_client.put_object(Bucket=bucket, Key=object_path, Body=file_io)
    except Exception:
        logging.error(f"failed to persist logs stream into S3 bucket {bucket}", exc_info=True)
        raise


def enrich_and_persist_checks_metadata(
        scan_reports: list[Report], s3_client: S3Client, bucket: str, full_repo_object_key: str
) -> dict[str, dict[str, str]]:
    """
    Save checks metadata into bridgecrew's platform
    :return:
    """
    checks_metadata_paths: dict[str, dict[str, str]] = {}
    for scan_report in scan_reports:
        check_type = scan_report.check_type
        checks_metadata_object = _extract_checks_metadata(scan_report, full_repo_object_key)
        checks_metadata_object_path = f'{full_repo_object_key}/{checkov_results_prefix}/{check_type}/checks_metadata.json'
        dpath.new(checks_metadata_paths, f"{check_type}/checks_metadata_path", checks_metadata_object_path)
        _put_json_object(s3_client, checks_metadata_object, bucket, checks_metadata_object_path)
    return checks_metadata_paths


def persist_graphs(graphs: dict[str, DiGraph | Graph], s3_client: S3Client, bucket: str, full_repo_object_key: str,
                   timeout: int, absolute_root_folder: str = '') -> None:
    def _upload_graph(check_type: str, graph: DiGraph | Graph, _absolute_root_folder: str = '') -> None:
        if isinstance(graph, DiGraph):
            json_obj = node_link_data(graph)
            graph_file_name = 'graph_networkx.json'
        elif isinstance(graph, Graph):
            json_obj = serialize_to_json(graph, _absolute_root_folder)
            graph_file_name = 'graph_igraph.json'
        else:
            logging.error(f"unsupported graph type '{graph.__class__.__name__}'")
            return
        s3_key = f'{graphs_repo_object_key}/{check_type}/{graph_file_name}'
        try:
            _put_json_object(s3_client, json_obj, bucket, s3_key)
        except Exception:
            logging.error(f'failed to upload graph from framework {check_type} to platform', exc_info=True)

    graphs_repo_object_key = full_repo_object_key.replace('checkov', 'graphs')[:-4]

    with futures.ThreadPoolExecutor() as executor:
        futures.wait(
            [executor.submit(_upload_graph, check_type, graph, absolute_root_folder) for
             check_type, graph in graphs.items()],
            return_when=futures.FIRST_EXCEPTION,
            timeout=timeout
        )
    logging.info(f"Done persisting {len(graphs)} graphs")



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, TypeVar, Generic

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block  # noqa
    from checkov.common.graph.graph_builder.local_graph import LocalGraph

T = TypeVar("T")
_Block = TypeVar("_Block", bound="Block")


class DBConnector(ABC, Generic[T]):
    @abstractmethod
    def save_graph(self, local_graph: LocalGraph[_Block]) -> T:
        pass

    @abstractmethod
    def get_reader_endpoint(self) -> T:
        pass

    @abstractmethod
    def get_writer_endpoint(self) -> T:
        pass

    @abstractmethod
    def disconnect(self) -> None:
        pass



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class K8SNodeGroupAutoUpgrade(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Kubernetes node group auto-upgrade is enabled."
        id = "CKV_YC_8"
        categories = (CheckCategories.GENERAL_SECURITY,)
        supported_resources = ("yandex_kubernetes_node_group",)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "maintenance_policy/[0]/auto_upgrade"

    def get_forbidden_values(self) -> list[Any]:
        return [False]


check = K8SNodeGroupAutoUpgrade()



from __future__ import annotations

import logging
from typing import List, Optional, Any

from checkov.common.output.common import SCADetails

UNFIXABLE_VERSION = "N/A"
OPEN_STATUS = "open"

TWISTCLI_TO_CHECKOV_LANG_NORMALIZATION = {
    "gem": "ruby",
    "nuget": "dotNet"
}


def get_file_path_for_record(rootless_file_path: str) -> str:
    return f"/{rootless_file_path}"


def get_resource_for_record(rootless_file_path: str, package_name: str) -> str:
    return f"{rootless_file_path}.{package_name}"


def get_package_alias(package_name: str, package_version: str) -> str:
    return f"{package_name}@{package_version}"


def get_license_policy_and_package_alias(policy: str, package_name: str) -> str:
    return f'{policy}_{package_name}'


def get_package_type(package_name: str, package_version: str, sca_details: SCADetails | None = None) -> str:
    if sca_details:
        return str(sca_details.package_types.get(f"{package_name}@{package_version}", ""))
    else:
        return ""


def get_registry_url(package: dict[str, Any]) -> str:
    if "registry" in package:
        return package.get("registry", "")
    return package.get("registryUrl", "")


def normalize_twistcli_language(language: str) -> str:
    """
    part of the language names that are returned by twistcli may be a little differ from those we use in checkov.
    this function's goal is to normalize them
    """
    return TWISTCLI_TO_CHECKOV_LANG_NORMALIZATION.get(language, language)


def get_package_lines(package: dict[str, Any]) -> list[int] | None:
    return package.get("linesNumbers", package.get("lines"))


def get_record_file_line_range(package: dict[str, Any], file_line_range: list[int] | None) -> list[int]:
    """
    Currently, there are 2 way for getting file_line_range for the sca-report:
    1. by the arg 'file_line_range' which comes from the runner - this is specific for entire file (e.g: image referencer)
    2. by a dedicated attribute in a package-object - (e.g: DT-cli V2)
    The purpose of this function is making sure there are no conflicts between those resources, and return a valid rage
    """
    package_line_range = get_package_lines(package)
    if package_line_range and file_line_range:
        logging.error(
            '[get_record_file_line_range] Both \'package_line_range\' and \'file_line_range\' are not None. Conflict.')
    return package_line_range or file_line_range or [0, 0]


def should_run_scan(runner_filter_checks: Optional[List[str]]) -> bool:
    return not (runner_filter_checks and all(
        not (check.startswith("CKV_CVE") or check.startswith("BC_CVE") or check.startswith("BC_LIC")) for check in
        runner_filter_checks))


def get_fix_version(vulnerability_details: dict[str, Any]) -> str:
    if "fix_version" in vulnerability_details:
        return str(vulnerability_details["fix_version"])

    if "lowest_fixed_version" in vulnerability_details:
        return str(vulnerability_details["lowest_fixed_version"])

    return UNFIXABLE_VERSION



from __future__ import annotations

import asyncio
import logging
import os
from typing import TYPE_CHECKING

import openai
from termcolor import colored

from checkov.common.bridgecrew.check_type import CheckType

if TYPE_CHECKING:
    from checkov.common.output.record import Record
    from typing_extensions import Self

OPENAI_MAX_FINDINGS = int(os.getenv("CKV_OPENAI_MAX_FINDINGS", 5))
OPENAI_MAX_TOKENS = int(os.getenv("CKV_OPENAI_MAX_TOKENS", 512))
OPENAI_MODEL = os.getenv("CKV_OPENAI_MODEL", "gpt-3.5-turbo")

RUNNER_DENY_LIST = {
    CheckType.POLICY_3D,
    CheckType.SCA_IMAGE,
    CheckType.SCA_PACKAGE,
    CheckType.SECRETS,
}


class OpenAi:
    _instance = None  # noqa: CCE003  # singleton

    def __new__(cls, api_key: str | None = None) -> Self:
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._should_run = True if api_key else False
            openai.api_key = api_key

        return cls._instance

    def enhance_records(self, runner_type: str, records: list[Record]) -> None:
        if not self._should_run:
            return
        if runner_type in RUNNER_DENY_LIST:
            return

        asyncio.run(self._generate_guidelines(records=records))

    async def _generate_guidelines(self, records: list[Record]) -> None:
        enhance_records = self._prioritize_findings(records=records)

        batches = [enhance_records]
        if len(enhance_records) > 20:
            # https://platform.openai.com/docs/guides/rate-limits/what-are-the-rate-limits-for-our-api
            # for free users 20 RPM is the limit, therefore splitting into batches of 10
            batch_size = 10
            batches = [records[i : i + batch_size] for i in range(0, len(enhance_records), batch_size)]

        self._print_warning(
            records_count=len(records),
            enhance_records_count=len(enhance_records),
            batches_count=len(batches),
        )

        for batch in batches:
            await asyncio.gather(*[self._chat_complete(record=record) for record in batch])

    async def _chat_complete(self, record: Record) -> None:
        if not record.code_block:
            # no need to ask OpenAI about guidelines, if we have no code blocks
            return

        try:
            completion = await openai.ChatCompletion.acreate(  # type:ignore[no-untyped-call]
                model=OPENAI_MODEL,
                messages=[
                    {"role": "system", "content": "You are a security tool"},
                    {
                        "role": "user",
                        "content": "".join(
                            [
                                f"fix following code, which violates checkov policy '{record.check_name}':\n",
                                *[line for _, line in record.code_block],
                            ]
                        ),
                    },
                    {"role": "user", "content": "Explain"},
                ],
                temperature=0,
                max_tokens=OPENAI_MAX_TOKENS,
            )
            logging.info(f"OpenAI request consumed {completion.usage.total_tokens} tokens")

            details = self._parse_completion_response(completion_content=completion.choices[0].message.content)
            if details:
                record.details = details
        except Exception:
            logging.info("Something went wrong while querying OpenAI", exc_info=True)

    def _prioritize_findings(self, records: list[Record]) -> list[Record]:
        if 0 < OPENAI_MAX_FINDINGS < len(records):
            # the higher severities should be preferred
            sorted_records = sorted(
                records, key=lambda record: record.severity.level if record.severity else 0, reverse=True  # type:ignore[has-type]
            )

            # to protect the user, just take the last x findings
            return sorted_records[-OPENAI_MAX_FINDINGS:]

        return records

    def _parse_completion_response(self, completion_content: str) -> list[str]:
        result = []

        if completion_content:
            result.append("The following text is AI generated and should be treated as a suggestion.")
            result.append("")

        in_code_block = False
        for line in completion_content.splitlines():
            if "```" in line:
                if in_code_block:
                    in_code_block = False
                else:
                    in_code_block = True
                continue
            if in_code_block:
                result.append(line)
            elif not line:
                result.append(line)
            else:
                result.extend(
                    sentence if sentence.endswith((".", ":")) else f"{sentence}."
                    for sentence in line.strip().split(". ")
                )

        return result

    def _print_warning(self, records_count: int, enhance_records_count: int, batches_count: int) -> None:
        max_findings_note = ""
        if 0 < OPENAI_MAX_FINDINGS < records_count:
            max_findings_note = (
                f"Found {records_count} failed checks and will provide enhanced guidelines for {OPENAI_MAX_FINDINGS}. To add enhanced guidelines for more findings,\n"
                "please adjust the env var 'CKV_OPENAI_MAX_FINDINGS' accordingly or set 0 to enhance all.\n"
            )

        print(
            colored(
                f"WARNING: About to request {enhance_records_count} enhanced guidelines and it may take up to {batches_count * 15}s.\n{max_findings_note}",
                "yellow",
            )
        )



from checkov.terraform.checks.resource.ncp.NACLInboundCheck import NACLInboundCheck


class NACLInbound3389(NACLInboundCheck):
    def __init__(self) -> None:
        super().__init__(check_id="CKV_NCP_11", port=3389)


check = NACLInbound3389()



from __future__ import annotations

import os
from typing import TYPE_CHECKING, Callable, Any

from checkov.common.images.image_referencer import Image
from checkov.common.util.str_utils import removeprefix
from checkov.dockerfile.utils import DOCKERFILE_STARTLINE, DOCKERFILE_ENDLINE

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction
    from typing_extensions import TypeAlias

_ExtractImagesCallableAlias: TypeAlias = Callable[["dict[str, Any]"], "list[str]"]


class DockerfileProvider:
    __slots__ = ("definitions",)

    def __init__(self, definitions: dict[str, dict[str, list[_Instruction]]]) -> None:
        self.definitions = definitions

    def extract_images_from_resources(self) -> list[Image]:
        images = []

        for file_path, config in self.definitions.items():
            instructions = config.get("FROM")
            if not isinstance(instructions, list):
                continue

            # just scan the last one
            instruction = instructions[-1]

            name = instruction["value"]

            if name.startswith("--platform"):
                # indicates a multi-platform build, therefore skip it
                # ex. FROM --platform=$BUILDPLATFORM golang:alpine AS build
                continue

            if " AS " in name:
                # indicates a multi-stage build, therefore remove everything starting from AS
                # ex. FROM amazonlinux:2 as run
                name = name.split(" AS ")[0]

            file_path = f'{removeprefix(file_path, os.getenv("BC_ROOT_DIR", ""))}'
            images.append(
                Image(
                    file_path=file_path,
                    name=name,
                    start_line=instruction[DOCKERFILE_STARTLINE] + 1,  # starts with 0
                    end_line=instruction[DOCKERFILE_ENDLINE] + 1,
                    related_resource_id=f'{file_path}:{file_path}.FROM',
                )
            )

        return images



from __future__ import annotations

from unittest import mock

from checkov.contributor_metrics import process_contributor, parse_gitlog


def test_process_contributor():
    contributor = 'Fake User <fake.user@gmail.com> (50):\n    commit-1667835804\n    commit-1667835527\n    ' \
                  'commit-1667834817\n    commit-1667826784\n    commit-1667808222'

    result = process_contributor(contributor)

    assert result == "Fake User <fake.user@gmail.com> 1667835804"


@mock.patch("subprocess.Popen")
def test_parse_gitlog(mock_subproc_popen):
    process_mock = mock.Mock()
    output = 'Fake User <fake1@paloaltonetworks.com> (40):\n      commit-1666516907\n      commit-1666259461\n      ' \
             'commit-1666259213\n      commit-1666258676\n      commit-1666258296\n      commit-1666258146\n      ' \
             'commit-1660669175\n      commit-1660668538\n      commit-1660626680\n      commit-1660626648\n      ' \
             'commit-1660589354\n      commit-1660588125\n\n' \
             'dependabot[bot] <49699333+dependabot[bot]@users.noreply.github.com> (38):\n      ' \
             'commit-1667819806\n      commit-1667819799\n      commit-1667819792\n      commit-1667216836\n       ' \
             'commit-1662541125\n      commit-1662476433\n      commit-1661850978\n      commit-1661850966\n       ' \
             'commit-1661746095\n      commit-1661154962\n      commit-1661154940\n\n' \
             'Fake User <fake2@paloaltonetworks.com> (31):\n      commit-1667216980\n      commit-1667216720\n      ' \
             'commit-1667216720\n      commit-1661265981\n      commit-1661265975\n      commit-1660808330\n      ' \
             'commit-1660799407\n      commit-1660737117\n      commit-1660632107\n      commit-1660631663'
    attrs = {"communicate.return_value": (output.encode('utf-8'), '')}
    process_mock.configure_mock(**attrs)
    mock_subproc_popen.return_value = process_mock

    result = parse_gitlog("my_repo", "jenkins")
    assert result == {"repository": "my_repo",
                      "source": "jenkins",
                      "contributors": ["Fake User <fake1@paloaltonetworks.com> 1666516907",
                                                                "dependabot[bot] <49699333+dependabot[bot]@users.noreply.github.com> 1667819806",
                                                                "Fake User <fake2@paloaltonetworks.com> 1667216980"],
                      'failedAttempts': []}



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class S3AbortIncompleteUploads(BaseResourceCheck):
    def __init__(self) -> None:
        """
        If you don't set this value in a lifecycle configuration you'll end up paying for s3
        resources you never could use
        """
        name = "Ensure S3 lifecycle configuration sets period for aborting failed uploads"
        id = "CKV_AWS_300"
        supported_resources = ("aws_s3_bucket_lifecycle_configuration",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["rule"]
        rules = conf.get("rule")
        if rules and isinstance(rules, list):
            for idx_rule, rule in enumerate(rules):
                if rule.get("abort_incomplete_multipart_upload") and rule.get("status") == ["Enabled"]:
                    self.evaluated_keys = [f"rule/[{idx_rule}]/abort_incomplete_multipart_upload"]
                    filter = rule.get("filter")
                    if filter and isinstance(filter, list) and filter[0]:
                        # it is possible to set an empty filter, which applies then to all objects
                        continue

                    return CheckResult.PASSED

        return CheckResult.FAILED


check = S3AbortIncompleteUploads()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_int


class AbsSecurityListUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port, is_exposed_by_default):
        name = f"Ensure no security list allow ingress from 0.0.0.0:0 to port {port}."
        supported_resources = ['oci_core_security_list']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port
        self.is_exposed_by_default = is_exposed_by_default

    def scan_resource_conf(self, conf):
        if 'ingress_security_rules' in conf:
            self.evaluated_keys = ['ingress_security_rules']
            rules = conf.get("ingress_security_rules", [])

            for idx, rule in enumerate(rules):
                if not isinstance(rule, list):
                    rule = [rule]
                for sub_rule_idx, sub_rule in enumerate(rule):
                    if "0.0.0.0/0" in sub_rule['source'][0] \
                            and (
                            (sub_rule['protocol'][0] != '1' and ('udp_options' not in sub_rule) and ('tcp_options' not in sub_rule))
                            or self.scan_protocol_conf(sub_rule, 'tcp_options', idx) != CheckResult.FAILED
                            or self.scan_protocol_conf(sub_rule, 'udp_options', idx) != CheckResult.FAILED
                            or sub_rule['protocol'][0] == 'all'):
                        self.evaluated_keys = [f'ingress_security_rules/[{sub_rule_idx}]/[{idx}]']
                        return CheckResult.FAILED

            return CheckResult.PASSED

        return CheckResult.FAILED if self.is_exposed_by_default else CheckResult.PASSED

    def scan_protocol_conf(self, rule, protocol_name, idx):
        """ scan udp/tcp_options configuration"""
        if protocol_name in rule:
            max_port = force_int(rule[protocol_name][0]['max'][0])
            min_port = force_int(rule[protocol_name][0]['min'][0])
            if min_port and max_port and min_port <= self.port <= max_port:
                return CheckResult.UNKNOWN
        self.evaluated_keys = [f'ingress_security_rules/[0]/[{idx}]/protocol/[0]/{protocol_name}']
        return CheckResult.FAILED



from __future__ import annotations

import logging
from typing import Optional, TYPE_CHECKING
from typing_extensions import TypedDict

if TYPE_CHECKING:
    from detect_secrets.core.potential_secret import PotentialSecret

PROHIBITED_FILES = ('Pipfile.lock', 'yarn.lock', 'package-lock.json', 'requirements.txt', 'go.sum')


ADDED = 'added'
REMOVED = 'removed'
GIT_HISTORY_OPTIONS = {ADDED, REMOVED}


CommitDiff = str


class RenamedFile(TypedDict):
    rename_from: str
    rename_to: str


class Commit:
    __slots__ = ("metadata", "files", "renamed_files")

    def __init__(
            self,
            metadata: CommitMetadata,
            files: dict[str, CommitDiff] | None = None,
            renamed_files: dict[str, RenamedFile] | None = None
    ):
        self.metadata: CommitMetadata = metadata
        self.files: dict[str, CommitDiff] = files or {}
        self.renamed_files: dict[str, RenamedFile] = renamed_files or {}

    def is_empty(self) -> bool:
        return not bool(self.files or self.renamed_files)

    def add_file(self, filename: str, commit_diff: CommitDiff) -> None:
        if self.files.get(filename):
            logging.warning(f'add_file-file {filename} already exist in commit')
            return
        self.files[filename] = commit_diff

    def rename_file(self, file_path: str, prev_filename: str, new_filename: str) -> None:
        if self.renamed_files.get(new_filename):
            logging.warning(f"rename_file-new filename {new_filename} was already renamed, might be an error")
            return
        self.renamed_files[file_path] = {
            'rename_from': prev_filename,
            'rename_to': new_filename
        }

    def remove_file(self, filename: str) -> None:
        if self.files.get(filename):
            del self.files[filename]


class CommitMetadata:
    __slots__ = ("commit_hash", "committer", "committed_datetime")

    def __init__(self, commit_hash: str, committer: str, committed_datetime: str):
        self.commit_hash: str = commit_hash
        self.committer: str = committer
        self.committed_datetime: str = committed_datetime


class EnrichedPotentialSecretMetadata(TypedDict, total=False):
    added_commit_hash: str
    removed_commit_hash: str
    code_line: Optional[str]
    added_by: Optional[str]
    removed_date: Optional[str]
    added_date: Optional[str]


class EnrichedPotentialSecret(EnrichedPotentialSecretMetadata):
    potential_secret: PotentialSecret  # noqa: CCE003  # a static attribute



from __future__ import annotations
from typing import Any

from checkov.circleci_pipelines.base_circleci_pipelines_check import BaseCircleCIPipelinesCheck
from checkov.common.models.enums import CheckResult
from checkov.yaml_doc.enums import BlockType


class SuspectCurlInScript(BaseCircleCIPipelinesCheck):
    def __init__(self) -> None:
        name = "Suspicious use of curl in run task"
        id = "CKV_CIRCLECIPIPELINES_7"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('jobs.*.steps[]',)
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not isinstance(conf, dict):
            return CheckResult.UNKNOWN, conf
        if "run" not in conf:
            return CheckResult.PASSED, conf
        run = conf.get("run", "")
        if type(run) == dict:
            run = run.get("command", "")
        if "curl" in run:
            badstuff = ['curl', 'POST']
            lines = run.split("\n")
            for line in lines:
                if all(x in line for x in badstuff):
                    return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = SuspectCurlInScript()



from __future__ import annotations

import logging
from collections import defaultdict
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Any, Optional, Dict, List

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import (
    integration as metadata_integration,
)
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.severities import Severities, Severity
from checkov.common.models.enums import CheckResult, ScanDataFormat
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.record import Record, DEFAULT_SEVERITY, SCA_PACKAGE_SCAN_CHECK_NAME, SCA_LICENSE_CHECK_NAME
from checkov.common.packaging import version as packaging_version
from checkov.common.sca.commons import (
    get_file_path_for_record,
    get_resource_for_record,
    get_package_alias,
    UNFIXABLE_VERSION,
    get_package_type,
    normalize_twistcli_language,
    get_registry_url, get_package_lines,
    get_record_file_line_range, get_license_policy_and_package_alias
)
from checkov.common.util.http_utils import request_wrapper
from checkov.runner_filter import RunnerFilter
from checkov.common.output.common import format_licenses_to_string

if TYPE_CHECKING:
    from aiohttp import ClientSession
    from checkov.common.output.common import SCADetails
    from checkov.common.output.report import Report
    from checkov.common.typing import _LicenseStatus, _CheckResult, _ScaSuppressions, _ScaSuppressionsMaps, \
        _SuppressedCves, _SuppressedLicenses


def create_report_license_record(
        rootless_file_path: str,
        file_abs_path: str,
        check_class: str,
        licenses_status: _LicenseStatus,
        package: dict[str, Any],
        sca_details: SCADetails | None = None,
        severity: Severity | None = None
) -> Record:
    package_name = licenses_status["package_name"]
    package_version = licenses_status["package_version"]
    policy = licenses_status["policy"]
    bc_status = licenses_status["status"]

    # renaming the status name from the one in platform's report to be convenient with checkov's-report
    status = "FAILED" if bc_status == "OPEN" else bc_status

    check_result: _CheckResult = {
        "result": CheckResult.FAILED,
    }
    if status == "COMPLIANT":
        check_result["result"] = CheckResult.PASSED

    code_block = get_code_block(package, package_name, package_version)

    details = {
        "package_name": package_name,
        "package_version": package_version,
        "package_registry": get_registry_url(package),
        "is_private_registry": package.get("isPrivateRegistry", False),
        "license": licenses_status["license"],
        "status": status,
        "policy": policy,
        "package_type": get_package_type(package_name, package_version, sca_details),
    }

    record = Record(
        check_id=policy,
        bc_check_id=policy,
        check_name=SCA_LICENSE_CHECK_NAME,
        check_result=check_result,
        code_block=code_block,
        file_path=get_file_path_for_record(rootless_file_path),
        file_line_range=get_package_lines(package) or [0, 0],
        resource=get_resource_for_record(rootless_file_path, package_name),
        check_class=check_class,
        evaluations=None,
        file_abs_path=file_abs_path,
        short_description=f"License {licenses_status['license']} - {package_name}: {package_version}",
        vulnerability_details=details,
        severity=severity
    )
    return record


def _update_details_by_scan_data_format(
        details: dict[str, Any],
        vulnerability_details: dict[str, Any],
        sca_details: SCADetails | None = None,
        scan_data_format: ScanDataFormat = ScanDataFormat.TWISTCLI
) -> None:
    if scan_data_format in {ScanDataFormat.TWISTCLI, ScanDataFormat.DEPENDENCY_TREE}:
        lowest_fixed_version = UNFIXABLE_VERSION
        package_version = vulnerability_details["packageVersion"]
        fixed_versions: list[packaging_version.Version | packaging_version.LegacyVersion] = []
        status = vulnerability_details.get("status") or "open"
        if status != "open":
            parsed_current_version = packaging_version.parse(package_version)
            for version in status.replace("fixed in", "").split(","):
                parsed_version = packaging_version.parse(version.strip())
                if parsed_version > parsed_current_version:
                    fixed_versions.append(parsed_version)

            if fixed_versions:
                lowest_fixed_version = str(min(fixed_versions))
        details.update({"status": status, "lowest_fixed_version": lowest_fixed_version,
                        "fixed_versions": fixed_versions, "image_details": sca_details})
    elif scan_data_format == ScanDataFormat.PLATFORM:
        status = vulnerability_details["status"]
        fix_version = vulnerability_details.get("cveStatus")
        details.update({"status": status, "fix_version": fix_version})


def get_code_block(package: dict[str, Any], package_name: str, package_version: str,
                   root_package: Optional[dict[str, Any]] = None) -> list[tuple[int, str]]:
    if root_package:
        root_lines_number = root_package.get("lines")
        root_code_block = root_package.get("code_block")
        if root_lines_number and root_code_block:
            return [(int(root_lines_number[0]), root_code_block)]

    lines_number = package.get("lines")
    code_block = package.get("code_block")

    if lines_number and code_block:
        return [(int(lines_number[0]), code_block)]

    return [(0, f"{package_name}: {package_version}")]


def get_fix_command_and_code(vulnerability_details: dict[str, Any], root_package: dict[str, Any] | None = None,
                             root_package_cve: dict[str, Any] | None = None) -> tuple[dict[str, Any] | None, str | None]:
    if root_package_cve:
        return root_package_cve.get('fixCommand'), root_package_cve.get('fixCode')

    if root_package and (
            root_package['name'] != vulnerability_details["packageName"] or root_package['version'] !=
            vulnerability_details["packageVersion"]):
        return None, None
    return vulnerability_details.get('fixCommand'), vulnerability_details.get('fixCode')


def get_package_lines_numbers(package: dict[str, Any], root_package: dict[str, Any] | None = None, file_line_range: list[int] | None = None) -> list[int]:
    if root_package:
        return get_record_file_line_range(root_package, file_line_range)
    return get_record_file_line_range(package, file_line_range)


def create_report_cve_record(
        rootless_file_path: str,
        file_abs_path: str,
        check_class: str,
        vulnerability_details: dict[str, Any],
        licenses: str,
        package: dict[str, Any],
        used_private_registry: bool = False,
        root_package: dict[str, Any] | None = None,
        runner_filter: RunnerFilter | None = None,
        sca_details: SCADetails | None = None,
        scan_data_format: ScanDataFormat = ScanDataFormat.TWISTCLI,
        file_line_range: list[int] | None = None,
        root_package_cve: dict[str, Any] | None = None
) -> Record:
    runner_filter = runner_filter or RunnerFilter()
    package_name = vulnerability_details["packageName"]
    package_version = vulnerability_details["packageVersion"]
    package_type = get_package_type(package_name, package_version, sca_details)
    cve_id = vulnerability_details.get("id", vulnerability_details.get("cveId", '')).upper()
    severity = vulnerability_details.get("severity", DEFAULT_SEVERITY)
    # sanitize severity names
    if severity == "moderate":
        severity = "medium"
    description = vulnerability_details.get("description")

    check_result: _CheckResult = {
        "result": CheckResult.FAILED,
    }

    if runner_filter.skip_cve_package and package_name in runner_filter.skip_cve_package:
        check_result = {
            "result": CheckResult.SKIPPED,
            "suppress_comment": f"Filtered by package '{package_name}'",
        }
    elif not runner_filter.within_threshold(Severities[severity.upper()]):
        check_result = {
            "result": CheckResult.SKIPPED,
            "suppress_comment": "Filtered by severity",
        }
    code_block = get_code_block(package, package_name, package_version, root_package)
    fix_command, fix_code = get_fix_command_and_code(vulnerability_details, root_package, root_package_cve)
    details = {
        "id": cve_id,
        "severity": severity,
        "package_name": package_name,
        "package_version": package_version,
        "package_registry": get_registry_url(package),
        "is_private_registry": package.get("isPrivateRegistry", False),
        "package_type": package_type,
        "link": vulnerability_details.get("link"),
        "cvss": vulnerability_details.get("cvss"),
        "vector": vulnerability_details.get("vector"),
        "description": description,
        "risk_factors": vulnerability_details.get("riskFactorsV2"),
        "published_date": vulnerability_details.get("publishedDate") or (datetime.now() - timedelta(
            days=vulnerability_details.get("publishedDays", 0))).isoformat(),
        "licenses": licenses,
        "root_package_name": root_package.get("name") if root_package else None,
        "root_package_version": root_package.get("version") if root_package else None,
        "root_package_file_line_range": get_package_lines(root_package) if root_package else None or [0, 0],
        "fix_command": fix_command
    }

    if used_private_registry:
        details["is_private_fix"] = vulnerability_details.get("isPrivateRegFix", False)

    if root_package_cve and root_package_cve.get('fixVersion'):
        details['root_package_fix_version'] = root_package_cve.get('fixVersion')

    _update_details_by_scan_data_format(details, vulnerability_details, sca_details, scan_data_format)
    record = Record(
        check_id=f"CKV_{cve_id.replace('-', '_')}",
        bc_check_id=f"BC_{cve_id.replace('-', '_')}",
        check_name=SCA_PACKAGE_SCAN_CHECK_NAME,
        check_result=check_result,
        code_block=code_block,
        file_path=get_file_path_for_record(rootless_file_path),
        file_line_range=get_package_lines_numbers(package, root_package, file_line_range),
        resource=get_resource_for_record(rootless_file_path, package_name),
        check_class=check_class,
        evaluations=None,
        file_abs_path=file_abs_path,
        severity=Severities[severity.upper()],
        description=description,
        short_description=f"{cve_id} - {package_name}: {package_version}",
        vulnerability_details=details,
    )

    record.fixed_definition = fix_code  # type: ignore
    return record


def _add_to_report_licenses_statuses(
        report: Report,
        check_class: str | None,
        scanned_file_path: str,
        rootless_file_path: str,
        runner_filter: RunnerFilter,
        packages_map: dict[str, dict[str, Any]],
        license_statuses: list[_LicenseStatus],
        sca_details: SCADetails | None = None,
        report_type: str | None = None,
        inline_suppressions_maps: _ScaSuppressionsMaps | None = None,
) -> dict[str, list[str]]:
    licenses_per_package_map: dict[str, list[str]] = defaultdict(list)

    for license_status in license_statuses:
        # filling 'licenses_per_package_map', will be used in the call to 'create_report_cve_record' for efficient
        # extracting of license per package
        package_name, package_version, license = (
            license_status["package_name"],
            license_status["package_version"],
            license_status["license"],
        )
        package_alias = get_package_alias(package_name, package_version)
        licenses_per_package_map[package_alias].append(license)

        policy = license_status["policy"]
        severity = metadata_integration.get_severity(policy)

        license_record = create_report_license_record(
            rootless_file_path=rootless_file_path,
            file_abs_path=scanned_file_path,
            check_class=check_class or "",
            licenses_status=license_status,
            package=packages_map.get(package_alias, {}),
            sca_details=sca_details,
            severity=severity
        )

        vulnerability_details = license_record.vulnerability_details or {}

        # apply inline suppressions
        suppressed = apply_licenses_inline_suppressions(
            record=license_record, vulnerability_details=vulnerability_details,
            inline_suppressions_maps=inline_suppressions_maps
        )

        if not suppressed and not runner_filter.should_run_check(
                check_id=policy,
                bc_check_id=policy,
                severity=severity,
                report_type=report_type,
        ):
            if runner_filter.checks:
                continue
            else:
                license_record.check_result = {
                    "result": CheckResult.SKIPPED,
                    "suppress_comment": f"{policy} is skipped",
                }

        report.add_resource(license_record.resource)
        report.add_record(license_record)

    return licenses_per_package_map


def get_inline_suppressions_map(inline_suppressions: _ScaSuppressions | None = None) -> _ScaSuppressionsMaps | None:
    if not inline_suppressions:
        return None
    suppressions_map: _ScaSuppressionsMaps = {}

    # fill cves suppressions map
    cve_suppresion_by_cve_map: dict[str, _SuppressedCves] = {}
    inline_suppressions_by_cve: list[_SuppressedCves] = inline_suppressions.get("cves", {}).get("byCve", [])
    for cve_suppression in inline_suppressions_by_cve:
        cve_id = cve_suppression.get("cveId")
        if cve_id:
            cve_suppresion_by_cve_map[cve_id] = cve_suppression

    # fill licenses suppressions map
    licenses_suppressions_by_policy_and_package_map: dict[str, _SuppressedLicenses] = {}
    inline_suppressions_by_license: list[_SuppressedLicenses] = inline_suppressions.get("licenses", {}).get("byPackage",
                                                                                                            [])
    for license_suppression in inline_suppressions_by_license:
        if license_suppression.get("licensePolicy") and license_suppression.get("packageName"):
            key = get_license_policy_and_package_alias(license_suppression["licensePolicy"],
                                                       license_suppression["packageName"])
            licenses_suppressions_by_policy_and_package_map[key] = license_suppression

    suppressions_map['cve_suppresion_by_cve_map'] = cve_suppresion_by_cve_map
    suppressions_map[
        'licenses_suppressions_by_policy_and_package_map'] = licenses_suppressions_by_policy_and_package_map

    return suppressions_map


def add_to_reports_cves_and_packages(
        report: Report,
        check_class: str | None,
        scanned_file_path: str,
        rootless_file_path: str,
        runner_filter: RunnerFilter,
        vulnerabilities: list[dict[str, Any]],
        packages: list[dict[str, Any]],
        packages_map: dict[str, dict[str, Any]],
        licenses_per_package_map: dict[str, list[str]],
        used_private_registry: bool = False,
        dependencies: dict[str, List[int]] | None = None,
        sca_details: SCADetails | None = None,
        report_type: str | None = None,
        inline_suppressions_maps: _ScaSuppressionsMaps | None = None,
        scan_data_format: ScanDataFormat = ScanDataFormat.TWISTCLI,
        file_line_range: list[int] | None = None
) -> None:
    is_dependency_tree_flow = bool(dependencies)

    vulnerable_packages, root_packages_list = create_vulnerable_packages_dict(vulnerabilities, packages,
                                                                              is_dependency_tree_flow)

    for package in packages:
        package_name, package_version = package["name"], package["version"]
        package_alias = get_package_alias(package_name, package_version)

        if package_alias in vulnerable_packages:
            package["cves"] = vulnerable_packages[package_alias]
        else:
            # adding resources without cves for adding them also in the output-bom-repors
            add_extra_resources_to_report(report, scanned_file_path, rootless_file_path,
                                          package, package_alias, licenses_per_package_map, sca_details)

    if is_dependency_tree_flow:
        add_to_reports_dependency_tree_cves(check_class, packages_map, licenses_per_package_map, packages, report,
                                            root_packages_list, rootless_file_path, runner_filter,
                                            scanned_file_path, used_private_registry, scan_data_format, sca_details,
                                            report_type, inline_suppressions_maps)
    else:  # twistlock scan results.
        for vulnerability in vulnerabilities:
            package_name, package_version = vulnerability["packageName"], vulnerability["packageVersion"]
            add_cve_record_to_report(vulnerability_details=vulnerability,
                                     package_name=package_name,
                                     package_version=package_version,
                                     packages_map=packages_map,
                                     rootless_file_path=rootless_file_path,
                                     scanned_file_path=scanned_file_path,
                                     check_class=check_class or "",
                                     licenses_per_package_map=licenses_per_package_map,
                                     runner_filter=runner_filter,
                                     sca_details=sca_details,
                                     scan_data_format=scan_data_format,
                                     report_type=report_type,
                                     report=report,
                                     inline_suppressions_maps=inline_suppressions_maps,
                                     file_line_range=file_line_range,
                                     used_private_registry=used_private_registry)


def add_to_reports_dependency_tree_cves(check_class: str | None, packages_map: dict[str, dict[str, Any]],
                                        licenses_per_package_map: dict[str, list[str]], packages: list[dict[str, Any]],
                                        report: Report, root_packages_list: list[int],
                                        rootless_file_path: str, runner_filter: RunnerFilter, scanned_file_path: str,
                                        used_private_registry: bool = False,
                                        scan_data_format: ScanDataFormat = ScanDataFormat.TWISTCLI,
                                        sca_details: SCADetails | None = None, report_type: str | None = None,
                                        inline_suppressions_maps: _ScaSuppressionsMaps | None = None) -> None:
    for root_package_index in root_packages_list:
        vulnerable_dependencies = find_vulnerable_dependencies(root_package_index, packages)

        root_package = packages[root_package_index]
        if len(root_package.get("cves", [])) > 0 or len(vulnerable_dependencies) > 0:
            root_package["vulnerable_dependencies"] = vulnerable_dependencies

        indirect_packages: dict[str, Any] = dict()
        for cve in root_package.get("cves", []):
            if 'causePackageName' in cve:
                cve_alias = f'{cve["cveId"]}@{cve["causePackageName"]}@{cve["causePackageVersion"]}'
                indirect_packages[cve_alias] = cve
                continue

            add_cve_record_to_report(vulnerability_details=cve, package_name=root_package['name'],
                                     package_version=root_package['version'], packages_map=packages_map,
                                     rootless_file_path=rootless_file_path, scanned_file_path=scanned_file_path,
                                     check_class=check_class, licenses_per_package_map=licenses_per_package_map,
                                     runner_filter=runner_filter, sca_details=sca_details,
                                     scan_data_format=scan_data_format, report_type=report_type, report=report,
                                     root_package=root_package, inline_suppressions_maps=inline_suppressions_maps,
                                     used_private_registry=used_private_registry)

        for dep in root_package.get("vulnerable_dependencies", []):
            for dep_cve in dep.get("cves", []):
                cve_alias = f'{dep_cve["cveId"]}@{dep_cve["packageName"]}@{dep_cve["packageVersion"]}'
                root_package_cve = None
                if cve_alias in indirect_packages:
                    root_package_cve = indirect_packages[cve_alias]

                add_cve_record_to_report(vulnerability_details=dep_cve, package_name=dep['name'],
                                         package_version=dep['version'], packages_map=packages_map,
                                         rootless_file_path=rootless_file_path, scanned_file_path=scanned_file_path,
                                         check_class=check_class, licenses_per_package_map=licenses_per_package_map,
                                         runner_filter=runner_filter, sca_details=sca_details,
                                         scan_data_format=scan_data_format, report_type=report_type, report=report,
                                         root_package=root_package,
                                         inline_suppressions_maps=inline_suppressions_maps,
                                         used_private_registry=used_private_registry, root_package_cve=root_package_cve)


def add_cve_record_to_report(vulnerability_details: dict[str, Any], package_name: str, package_version: str,
                             packages_map: dict[str, dict[str, Any]], rootless_file_path: str,
                             scanned_file_path: str, check_class: Optional[str],
                             licenses_per_package_map: dict[str, list[str]], runner_filter: RunnerFilter,
                             sca_details: Optional[SCADetails], scan_data_format: ScanDataFormat,
                             report_type: Optional[str], report: Report, used_private_registry: bool = False,
                             root_package: dict[str, Any] | None = None,
                             inline_suppressions_maps: _ScaSuppressionsMaps | None = None,
                             file_line_range: list[int] | None = None,
                             root_package_cve: dict[str, Any] | None = None) -> None:
    package_alias = get_package_alias(package_name, package_version)
    cve_record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=scanned_file_path,
        check_class=check_class or "",
        vulnerability_details=vulnerability_details,
        licenses=format_licenses_to_string(licenses_per_package_map[package_alias]),
        package=packages_map.get(package_alias, {}),
        runner_filter=runner_filter,
        sca_details=sca_details,
        scan_data_format=scan_data_format,
        root_package=root_package,
        root_package_cve=root_package_cve,
        file_line_range=file_line_range,
        used_private_registry=used_private_registry
    )
    suppressed = apply_cves_inline_suppressions(
        record=cve_record, inline_suppressions_maps=inline_suppressions_maps
    )

    if not suppressed and not runner_filter.should_run_check(
            check_id=cve_record.check_id,
            bc_check_id=cve_record.bc_check_id,
            severity=cve_record.severity,
            report_type=report_type,
    ):
        if runner_filter.checks:
            return
        else:
            cve_record.check_result = {
                "result": CheckResult.SKIPPED,
                "suppress_comment": f"{vulnerability_details.get('cveId', vulnerability_details.get('id', ''))} is skipped"
            }

    report.add_resource(cve_record.resource)
    report.add_record(cve_record)


def apply_cves_inline_suppressions(
        record: Record, inline_suppressions_maps: _ScaSuppressionsMaps | None = None
) -> bool:
    """Applies the inline suppression and returns an accomplish status"""

    if inline_suppressions_maps and record.vulnerability_details and inline_suppressions_maps.get(
            "cve_suppresion_by_cve_map"):
        cve_id = record.vulnerability_details.get("id", "")
        cve_suppression = inline_suppressions_maps["cve_suppresion_by_cve_map"].get(cve_id)
        if cve_suppression:
            record.check_result = {
                "result": CheckResult.SKIPPED,
                "suppress_comment": cve_suppression.get('reason', ''),
            }
            return True

    return False


def apply_licenses_inline_suppressions(
        record: Record, vulnerability_details: dict[str, Any],
        inline_suppressions_maps: _ScaSuppressionsMaps | None = None
) -> bool:
    """Applies the inline suppression and returns an accomplish status"""

    if inline_suppressions_maps and inline_suppressions_maps.get("licenses_suppressions_by_policy_and_package_map"):
        key = get_license_policy_and_package_alias(vulnerability_details.get("policy", ""),
                                                   vulnerability_details.get("package_name", ""))
        license_suppression = inline_suppressions_maps["licenses_suppressions_by_policy_and_package_map"].get(key)
        if license_suppression:
            record.check_result = {
                "result": CheckResult.SKIPPED,
                "suppress_comment": license_suppression.get('reason', ''),
            }
            return True

    return False


def find_vulnerable_dependencies(root_package_index: int, packages: list[dict[str, Any]]) -> list[dict[str, Any]]:
    vulnerable_dependencies: list[dict[str, Any]] = []
    for vulnerable_dependency_idx in packages[root_package_index].get('vulnerable_dependencies', []):
        vulnerable_dependencies.append(packages[vulnerable_dependency_idx])
    return vulnerable_dependencies


def create_root_packages_list(root_packages_list: list[int], packages: list[dict[str, Any]], package: dict[str, Any],
                              dependencies: Optional[Dict[str, List[int]]]) -> None:
    if dependencies:
        if package.get("root", ""):
            root_packages_list.append(packages.index(package))
    else:
        # if we don't have dependencies, all packages will be "roots"
        root_packages_list.append(packages.index(package))


def create_vulnerable_packages_dict(vulnerabilities: list[dict[str, Any]], packages: list[dict[str, Any]],
                                    is_dependency_tree_flow: bool) -> tuple[dict[str, list[dict[str, Any]]], list[int]]:
    vulnerable_packages: dict[str, list[dict[str, Any]]] = dict()
    root_packages_list: list[int] = []
    if is_dependency_tree_flow:
        for package_idx, package in enumerate(packages):
            if package.get("root", False):
                root_packages_list.append(package_idx)

            package_alias = get_package_alias(package["name"], package["version"])
            for cve_idx in package.get('cves_index', []):
                vulnerable_packages.setdefault(package_alias, []).append(vulnerabilities[cve_idx])
    else:
        for vulnerability in vulnerabilities:
            package_alias = get_package_alias(vulnerability["packageName"], vulnerability["packageVersion"])
            vulnerable_packages.setdefault(package_alias, []).append(vulnerability)

    return vulnerable_packages, root_packages_list


def add_extra_resources_to_report(report: Report, scanned_file_path: str, rootless_file_path: str,
                                  package: dict[str, Any], package_alias: str,
                                  licenses_per_package_map: dict[str, list[str]],
                                  sca_details: Optional[SCADetails]) -> None:
    package_name, package_version = package["name"], package["version"]
    report.extra_resources.add(
        ExtraResource(
            file_abs_path=scanned_file_path,
            file_path=get_file_path_for_record(rootless_file_path),
            resource=get_resource_for_record(rootless_file_path, package_name),
            file_line_range=get_package_lines(package),
            vulnerability_details={
                "package_name": package_name,
                "package_version": package_version,
                "package_registry": get_registry_url(package),
                "is_private_registry": package.get("isPrivateRegistry", False),
                "licenses": format_licenses_to_string(
                    licenses_per_package_map[package_alias]),
                "package_type": get_package_type(package_name, package_version, sca_details)
            },
        )
    )


def add_to_report_sca_data(
        report: Report,
        check_class: str | None,
        scanned_file_path: str,
        rootless_file_path: str,
        runner_filter: RunnerFilter,
        vulnerabilities: list[dict[str, Any]],
        packages: list[dict[str, Any]],
        license_statuses: list[_LicenseStatus],
        used_private_registry: bool = False,
        dependencies: dict[str, list[int]] | None = None,
        sca_details: SCADetails | None = None,
        report_type: str | None = None,
        inline_suppressions: _ScaSuppressions | None = None,
        file_line_range: list[int] | None = None
) -> None:
    inline_suppressions_maps: _ScaSuppressionsMaps | None = get_inline_suppressions_map(inline_suppressions)
    packages_map: dict[str, dict[str, Any]] = {get_package_alias(p["name"], p["version"]): p for p in packages}
    licenses_per_package_map: dict[str, list[str]] = \
        _add_to_report_licenses_statuses(report, check_class, scanned_file_path, rootless_file_path, runner_filter,
                                         packages_map, license_statuses, sca_details, report_type,
                                         inline_suppressions_maps)
    # if dependencies is empty list it means we got results via DependencyTree scan but no dependencies have found.
    add_to_reports_cves_and_packages(report=report, check_class=check_class,
                                     scanned_file_path=scanned_file_path,
                                     rootless_file_path=rootless_file_path,
                                     runner_filter=runner_filter,
                                     vulnerabilities=vulnerabilities,
                                     packages=packages,
                                     packages_map=packages_map,
                                     licenses_per_package_map=licenses_per_package_map,
                                     sca_details=sca_details,
                                     report_type=report_type,
                                     scan_data_format=ScanDataFormat.DEPENDENCY_TREE,
                                     dependencies=dependencies,
                                     inline_suppressions_maps=inline_suppressions_maps,
                                     file_line_range=file_line_range,
                                     used_private_registry=used_private_registry)


def _get_request_input(packages: list[dict[str, Any]]) -> list[dict[str, Any]]:
    return [
        {"name": package.get("name", ""), "version": package.get("version", ""),
         "lang": normalize_twistcli_language(package.get("type", ""))}
        for package in packages
    ]


def get_license_statuses(packages: list[dict[str, Any]]) -> list[_LicenseStatus]:
    requests_input = _get_request_input(packages)
    if not requests_input:
        return []
    try:
        response = request_wrapper(
            method="POST",
            url=f"{bc_integration.api_url}/api/v1/vulnerabilities/packages/get-licenses-violations",
            headers=bc_integration.get_default_headers("POST"),
            json={"packages": requests_input},
            should_call_raise_for_status=True
        )
        response_json = response.json()
        license_statuses: list[_LicenseStatus] = _extract_license_statuses(response_json)
        return license_statuses
    except Exception:
        error_message = (
            "failing when trying to get licenses-violations. it is apparently some unexpected "
            "connection issue. please try later. in case it keep happening. please report."
        )
        logging.info(error_message, exc_info=True)

    return []


async def get_license_statuses_async(session: ClientSession, packages: list[dict[str, Any]], image_name: str) \
        -> dict[str, str | list[_LicenseStatus]]:
    """
    This is an async implementation of `get_license_statuses`. The only change is we're getting a session
    as an input, and the asyncio behavior is managed in the calling method.
    """
    requests_input = _get_request_input(packages)
    url = f"{bc_integration.api_url}/api/v1/vulnerabilities/packages/get-licenses-violations"
    if not requests_input:
        return {'image_name': image_name, 'licenses': []}
    try:
        async with session.request("POST", url, headers=bc_integration.get_default_headers("POST"),
                                   json={"packages": requests_input}) as resp:
            response_json = await resp.json()

        license_statuses = _extract_license_statuses(response_json)
        return {'image_name': image_name, 'licenses': license_statuses}
    except Exception as e:
        error_message = (
            "failing when trying to get licenses-violations. it is apparently some unexpected "
            "connection issue. please try later. in case it keeps happening, please report."
            f"Error: {str(e)}"
        )
        logging.info(error_message, exc_info=True)

        return {'image_name': image_name, 'licenses': []}


def _extract_license_statuses(response_json: dict[str, list[dict[str, str]]]) -> list[_LicenseStatus]:
    license_statuses: list[_LicenseStatus] = [
        {
            "package_name": license_violation.get("name", ""),
            "package_version": license_violation.get("version", ""),
            "policy": license_violation.get("policy", "BC_LIC1"),
            "license": license_violation.get("license", ""),
            "status": license_violation.get("status", "COMPLIANT")
        }
        for license_violation in response_json.get("violations", [])
    ]
    return license_statuses



from __future__ import annotations
import abc
from typing import Any

from checkov.policies_3d.syntax.syntax import Predicate


class CVEPredicate(Predicate):
    def __init__(self, cve_report: dict[str, Any]) -> None:
        super().__init__()
        self.cve_report = cve_report

    @abc.abstractmethod
    def __call__(self) -> bool:
        raise NotImplementedError()


class RiskFactorCVEContains(CVEPredicate):
    def __init__(self, risk_factors: list[str], cve_report: dict[str, Any]) -> None:
        super().__init__(cve_report)
        self.risk_factors = [rf.lower() for rf in risk_factors]
        report_risk_factors = cve_report.get('riskFactors', []) or []
        if isinstance(report_risk_factors, str):
            report_risk_factors = [report_risk_factors]

        self.cve_report['riskFactors'] = [rf.lower() for rf in report_risk_factors]

    def __call__(self) -> bool:
        self.is_true = all(rf in self.cve_report['riskFactors'] for rf in self.risk_factors)

        if not self.is_true:
            for rf in self.cve_report['riskFactors']:
                self.is_true = all(rf.startswith(predicate_rf) for predicate_rf in self.risk_factors)
                if self.is_true:
                    break

        return self.is_true

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, RiskFactorCVEContains):
            return False

        return set(self.risk_factors) == set(other.risk_factors) and self.cve_report['cveId'] == other.cve_report['cveId']

    def __hash__(self) -> Any:
        return hash(('risk_factors', tuple(self.risk_factors), 'cveId', self.cve_report['cveId']))



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceMinTLSVersion(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure web app is using the latest version of TLS encryption"
        id = "CKV_AZURE_15"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        if self.entity_type == "azurerm_app_service":
            return "site_config/[0]/min_tls_version/[0]"
        else:
            return "site_config/[0]/minimum_tls_version/[0]"

    def get_expected_value(self):
        return '1.2'


check = AppServiceMinTLSVersion()



from __future__ import annotations

import concurrent.futures
import logging
import re
import json
from typing import List, Tuple, Dict, Any, Optional, Pattern, TYPE_CHECKING

from igraph import Graph
from bc_jsonpath_ng.ext import parse

from checkov.common.graph.checks_infra import debug
from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

from concurrent.futures import ThreadPoolExecutor

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.common.util.var_utils import is_terraform_variable_dependent
from checkov.terraform.graph_builder.graph_components.block_types import BlockType as TerraformBlockType

if TYPE_CHECKING:
    from bc_jsonpath_ng import JSONPath
    from checkov.common.typing import LibraryGraph

SUPPORTED_BLOCK_TYPES = {BlockType.RESOURCE, TerraformBlockType.DATA, TerraformBlockType.MODULE, TerraformBlockType.PROVIDER}
WILDCARD_PATTERN = re.compile(r"(\S+[.][*][.]*)+")


class BaseAttributeSolver(BaseSolver):
    operator = ""  # noqa: CCE003  # a static attribute
    is_value_attribute_check = True  # noqa: CCE003  # a static attribute
    jsonpath_parsed_statement_cache: "dict[str, JSONPath]" = {}  # noqa: CCE003  # global cache

    def __init__(
        self, resource_types: List[str], attribute: Optional[str], value: Any, is_jsonpath_check: bool = False
    ) -> None:
        super().__init__(SolverType.ATTRIBUTE)
        self.resource_types = resource_types
        self.attribute = attribute
        self.value = value
        self.is_jsonpath_check = is_jsonpath_check

    def run(self, graph_connector: LibraryGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        executer = ThreadPoolExecutor()
        jobs = []
        passed_vertices: List[Dict[str, Any]] = []
        failed_vertices: List[Dict[str, Any]] = []
        unknown_vertices: List[Dict[str, Any]] = []
        if isinstance(graph_connector, Graph):
            if self.resource_types:
                select_kwargs = {"resource_type_in": list(self.resource_types)}
            else:
                select_kwargs = {"block_type__in": list(SUPPORTED_BLOCK_TYPES)}

            for data in graph_connector.vs.select(**select_kwargs)["attr"]:
                result = self.get_operation(vertex=data)
                # A None indicate for UNKNOWN result - the vertex shouldn't be added to the passed or the failed vertices
                if result is None:
                    unknown_vertices.append(data)
                elif result:
                    passed_vertices.append(data)
                else:
                    failed_vertices.append(data)

            return passed_vertices, failed_vertices, unknown_vertices

        for _, data in graph_connector.nodes(data=True):
            if (not self.resource_types or data.get(CustomAttributes.RESOURCE_TYPE) in self.resource_types) \
                    and data.get(CustomAttributes.BLOCK_TYPE) in SUPPORTED_BLOCK_TYPES:
                jobs.append(executer.submit(
                    self._process_node, data, passed_vertices, failed_vertices, unknown_vertices))

        concurrent.futures.wait(jobs)
        return passed_vertices, failed_vertices, unknown_vertices

    def get_operation(self, vertex: Dict[str, Any]) -> Optional[bool]:
        # if this value contains an underendered variable, then we cannot evaluate value checks,
        # and will return None (for UNKNOWN)
        # handle edge cases in some policies that explicitly look for blank values
        # we also need to check the attribute stack - e.g., if they are looking for tags.component, but tags = local.tags,
        # then we actually need to see if tags is variable dependent as well
        attr_parts = self.attribute.split('.')  # type:ignore[union-attr]  # due to attribute can be None (but not really)
        attr_to_check = None
        for attr in attr_parts:
            attr_to_check = f'{attr_to_check}.{attr}' if attr_to_check else attr
            value_to_check = vertex.get(attr_to_check)
            value_to_check = self._render_json_str(value_to_check)

            # we can only check is_attribute_value_check when evaluating the full attribute
            # for example, if we have a policy that says "tags.component exists", and tags = local.tags, then
            # we need to check if tags is variable dependent even though this is a not value_attribute check
            if (attr_to_check != self.attribute or self.is_value_attribute_check) \
                    and self._is_variable_dependant(value_to_check, vertex['source_']) \
                    and self.value != '':
                return None

        if self.attribute and (self.is_jsonpath_check or re.match(WILDCARD_PATTERN, self.attribute)):
            attribute_matches = self.get_attribute_matches(vertex)
            filtered_attribute_matches = attribute_matches
            if self.is_value_attribute_check and self.value != '':
                filtered_attribute_matches = []
                for attribute in attribute_matches:
                    resource_variable_dependant = self._is_variable_dependant(vertex.get(attribute), vertex['source_'])
                    policy_variable_dependant = self._is_variable_dependant(self.value, vertex['source_'])
                    if not resource_variable_dependant or resource_variable_dependant and policy_variable_dependant:
                        filtered_attribute_matches.append(attribute)
            if attribute_matches:
                result = self._evaluate_attribute_matches(
                    vertex=vertex,
                    attribute_matches=attribute_matches,
                    filtered_attribute_matches=filtered_attribute_matches,
                )
                if result is not None:
                    # skip unknown
                    debug.attribute_block(
                        resource_types=self.resource_types,
                        attribute=self.attribute,
                        operator=self.operator,
                        value=self.value,
                        resource=vertex,
                        status="passed" if result is True else "failed",
                    )

                return result

        result = self.resource_type_pred(vertex, self.resource_types) and self._get_operation(
            vertex=vertex, attribute=self.attribute
        )

        debug.attribute_block(
            resource_types=self.resource_types,
            attribute=self.attribute,
            operator=self.operator,
            value=self.value,
            resource=vertex,
            status="passed" if result is True else "failed",
        )

        return result

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        raise NotImplementedError

    def _process_node(
        self, data: Dict[str, Any], passed_vartices: List[Dict[str, Any]], failed_vertices: List[Dict[str, Any]],
            unknown_vertices: List[Dict[str, Any]]
    ) -> None:
        if not self.resource_type_pred(data, self.resource_types):
            return
        result = self.get_operation(vertex=data)
        # A None indicate for UNKNOWN result - the vertex shouldn't be added to the passed or the failed vertices
        if result is None:
            unknown_vertices.append(data)
        elif result:
            passed_vartices.append(data)
        else:
            failed_vertices.append(data)

    def _evaluate_attribute_matches(
        self, vertex: dict[str, Any], attribute_matches: list[str], filtered_attribute_matches: list[str]
    ) -> bool | None:
        if self.is_jsonpath_check:
            if self.resource_type_pred(vertex, self.resource_types) and all(
                self._get_operation(vertex=vertex, attribute=attr) for attr in filtered_attribute_matches
            ):
                return True if len(attribute_matches) == len(filtered_attribute_matches) else None
            return False

        if self.resource_type_pred(vertex, self.resource_types) and any(
            self._get_operation(vertex=vertex, attribute=attr) for attr in filtered_attribute_matches
        ):
            return True
        return False if len(attribute_matches) == len(filtered_attribute_matches) else None

    def get_attribute_matches(self, vertex: Dict[str, Any]) -> List[str]:
        try:
            attribute_matches: List[str] = []
            if self.is_jsonpath_check and self.attribute:
                parsed_attr = self._get_cached_jsonpath_statement(statement=self.attribute)

                for match in parsed_attr.find(vertex):
                    full_path = str(match.full_path)
                    if full_path not in vertex:
                        vertex[full_path] = match.value

                    attribute_matches.append(full_path)
            elif isinstance(self.attribute, str):
                attribute_patterns = self.get_attribute_patterns(self.attribute)
                attribute_parts = [attr for attr in self.attribute.split(".") if attr != "*"]
                for attr in vertex:
                    if any(part not in attr for part in attribute_parts):
                        # if even one attribute part doesn't exist in the vertex attribute, then no need to further proceed
                        continue
                    if any(re.match(attribute_pattern, attr) for attribute_pattern in attribute_patterns):
                        attribute_matches.append(attr)

            return attribute_matches
        except Exception:
            logging.debug('Error parsing or evaluating jsonpath expression', exc_info=True)
            raise

    @staticmethod
    def get_attribute_patterns(attribute: str) -> Tuple[Pattern[str], Pattern[str]]:
        index_pattern = r"[\d]+"
        split_by_dots = attribute.split(".")

        pattern_parts = []
        pattern_parts_without_index = []
        for attr_part in split_by_dots:
            if attr_part == "*":
                pattern_parts.append(index_pattern)
            else:
                attr_part_pattern = f"({attr_part})"
                pattern_parts.append(attr_part_pattern)
                pattern_parts_without_index.append(attr_part_pattern)

        pattern = f'^{"[.]".join(pattern_parts)}$'
        pattern_with_index = re.compile(pattern)

        pattern = f'^{"[.]".join(pattern_parts_without_index)}$'
        pattern_without_index = re.compile(pattern)

        return pattern_with_index, pattern_without_index

    @staticmethod
    def _is_variable_dependant(value: Any, source: str) -> bool:
        if source.lower() == 'terraform' and is_terraform_variable_dependent(value):
            return True
        # TODO add logic for CloudFormation
        # elif source == 'CloudFormation' and is_cloudformation_variable_dependent(value):
        #     return True

        return False

    @staticmethod
    def _render_json_str(value_to_check: Any) -> Any:
        """
        Tries to render objects containing json dict, for example:
        - resource "aws_iam_policy" "example1" {
            policy == <<POLICY
                {
                ... json-data
                }
            POLICY
        }
        - resource "aws_emr_security_configuration" "example2" {
            configuration = <<EOF
                {
                ... json-data
                }
            EOF
        }
        """
        try:
            # Check if the value looks like a json object, and if it is try to parse it
            if isinstance(value_to_check, str) and value_to_check.strip().startswith('{'):
                value_to_check = json.loads(value_to_check)
                return value_to_check
        except Exception as e:
            logging.info(f'cant parse policy str to object, {str(e)}')
        return value_to_check

    def _get_cached_jsonpath_statement(self, statement: str) -> JSONPath:
        """Returns the parsed jsonpath statement from the cache or adds it"""

        if statement not in BaseAttributeSolver.jsonpath_parsed_statement_cache:
            parsed_attr = parse(statement)
            BaseAttributeSolver.jsonpath_parsed_statement_cache[statement] = parsed_attr
            return parsed_attr

        return BaseAttributeSolver.jsonpath_parsed_statement_cache[statement]



from __future__ import annotations

import json
import logging
import typing
from abc import abstractmethod
from typing import Dict, List, Any, Union

from checkov.common.models.enums import CheckResult

if typing.TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class BaseTerraformCloudsplainingIAMScanner:
    # creating a PolicyDocument is computational expensive,
    # therefore a cache is defined at class level
    policy_document_cache: Dict[str, PolicyDocument] = {}  # noqa: CCE003

    def scan_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if self.should_scan_conf(conf):
            try:
                if self.cache_key not in BaseTerraformCloudsplainingIAMScanner.policy_document_cache.keys():
                    policy = self.convert_to_iam_policy(conf)
                    BaseTerraformCloudsplainingIAMScanner.policy_document_cache[self.cache_key] = policy
                violations = self.cloudsplaining_analysis(
                    BaseTerraformCloudsplainingIAMScanner.policy_document_cache[self.cache_key]
                )
            except Exception:
                # this might occur with templated iam policies where ARN is not in place or similar
                logging.debug(f"could not run cloudsplaining analysis on policy {conf}")
                return CheckResult.UNKNOWN
            if violations:
                logging.debug(f"detailed cloudsplainging finding: {json.dumps(violations, indent=2, default=str)}")
                return CheckResult.FAILED
        return CheckResult.PASSED

    @property
    @abstractmethod
    def cache_key(self) -> str:
        pass

    @abstractmethod
    def should_scan_conf(self, conf: Dict[str, List[Any]]) -> bool:
        pass

    @abstractmethod
    def convert_to_iam_policy(self, conf: Dict[str, List[Any]]) -> PolicyDocument:
        pass

    @abstractmethod
    def cloudsplaining_analysis(self, policy: PolicyDocument) -> Union[List[str], List[Dict[str, Any]]]:
        raise NotImplementedError()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class LambdaFunctionURLAuth(BaseResourceNegativeValueCheck):

    def __init__(self):
        name = "Ensure that Lambda function URLs AuthType is not None"
        id = "CKV_AWS_258"
        supported_resources = ['aws_lambda_function_url']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "authorization_type"

    def get_forbidden_values(self):
        return ["NONE"]


check = LambdaFunctionURLAuth()



from checkov.terraform.checks.resource.ncp.NACLInboundCheck import NACLInboundCheck


class NACLInbound20(NACLInboundCheck):
    def __init__(self) -> None:
        super().__init__(check_id="CKV_NCP_8", port=20)


check = NACLInbound20()



from __future__ import annotations

import argparse
import json
import logging
import os
from collections.abc import Iterable

from typing import List, Dict, Union, Any, Optional, TYPE_CHECKING, cast
from colorama import init
from junit_xml import TestCase, TestSuite, to_xml_report_string
from tabulate import tabulate
from termcolor import colored

from checkov.common.bridgecrew.code_categories import CodeCategoryType
from checkov.common.bridgecrew.severities import BcSeverities, Severity
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult, ErrorStatus
from checkov.common.output.ai import OpenAi
from checkov.common.typing import _ExitCodeThresholds, _ScaExitCodeThresholds
from checkov.common.output.record import Record, SCA_PACKAGE_SCAN_CHECK_NAME
from checkov.common.util.consts import PARSE_ERROR_FAIL_FLAG, CHECKOV_RUN_SCA_PACKAGE_SCAN_V2
from checkov.common.util.json_utils import CustomJSONEncoder
from checkov.runner_filter import RunnerFilter

from checkov.sca_package_2.output import create_cli_output as create_sca_package_cli_output_v2

from checkov.sca_package.output import create_cli_output as create_sca_package_cli_output_v1

from checkov.policies_3d.output import create_cli_output as create_3d_policy_cli_output

from checkov.version import version

if TYPE_CHECKING:
    from checkov.common.output.baseline import Baseline
    from checkov.common.output.extra_resource import ExtraResource

init(autoreset=True)

SEVERITY_TO_SARIF_LEVEL = {
    "critical": "error",
    "high": "error",
    "medium": "warning",
    "low": "note",
    "none": "none",
}


class Report:
    def __init__(self, check_type: str):
        self.check_type = check_type
        self.passed_checks: list[Record] = []
        self.failed_checks: list[Record] = []
        self.skipped_checks: list[Record] = []
        self.parsing_errors: list[str] = []
        self.resources: set[str] = set()
        self.extra_resources: set[ExtraResource] = set()
        self.image_cached_results: List[dict[str, Any]] = []
        self.error_status: ErrorStatus = ErrorStatus.SUCCESS

    def set_error_status(self, error_status: ErrorStatus) -> None:
        self.error_status = error_status

    def add_parsing_errors(self, errors: "Iterable[str]") -> None:
        for file in errors:
            self.add_parsing_error(file)

    def add_parsing_error(self, file: str) -> None:
        if file:
            self.parsing_errors.append(file)

    def add_resource(self, resource: str) -> None:
        self.resources.add(resource)

    def add_record(self, record: Record) -> None:
        if record.check_result["result"] == CheckResult.PASSED:
            self.passed_checks.append(record)
        if record.check_result["result"] == CheckResult.FAILED:
            self.failed_checks.append(record)
        if record.check_result["result"] == CheckResult.SKIPPED:
            self.skipped_checks.append(record)

    def get_summary(self) -> Dict[str, Union[int, str]]:
        return {
            "passed": len(self.passed_checks),
            "failed": len(self.failed_checks),
            "skipped": len(self.skipped_checks),
            "parsing_errors": len(self.parsing_errors),
            "resource_count": len(self.resources),
            "checkov_version": version,
        }

    def get_json(self) -> str:
        return json.dumps(self.get_dict(), indent=4, cls=CustomJSONEncoder)

    def get_all_records(self) -> List[Record]:
        return self.failed_checks + self.passed_checks + self.skipped_checks

    def get_dict(self, is_quiet: bool = False, url: str | None = None, full_report: bool = False) -> dict[str, Any]:
        if not url:
            url = "Add an api key '--bc-api-key <api-key>' to see more detailed insights via https://bridgecrew.cloud"
        if is_quiet:
            return {
                "check_type": self.check_type,
                "results": {
                    "failed_checks": [check.__dict__ for check in self.failed_checks]
                },
                "summary": self.get_summary(),
            }
        if full_report:
            return {
                "check_type": self.check_type,
                "checks": {
                    "passed_checks": [check.__dict__ for check in self.passed_checks],
                    "failed_checks": [check.__dict__ for check in self.failed_checks],
                    "skipped_checks": [check.__dict__ for check in self.skipped_checks]
                },
                "image_cached_results": [res.__dict__ for res in self.image_cached_results]
            }
        else:
            return {
                "check_type": self.check_type,
                "results": {
                    "passed_checks": [check.__dict__ for check in self.passed_checks],
                    "failed_checks": [check.__dict__ for check in self.failed_checks],
                    "skipped_checks": [check.__dict__ for check in self.skipped_checks],
                    "parsing_errors": list(self.parsing_errors),
                },
                "summary": self.get_summary(),
                "url": url,
            }

    def get_exit_code(self, exit_code_thresholds: Union[_ExitCodeThresholds, _ScaExitCodeThresholds]) -> int:
        """
        Returns the appropriate exit code depending on the flags that are passed in.

        :return: Exit code 0 or 1.
        """

        hard_fail_on_parsing_errors = os.getenv(PARSE_ERROR_FAIL_FLAG, "false").lower() == 'true'
        logging.debug(f'In get_exit_code; exit code thresholds: {exit_code_thresholds}, hard_fail_on_parsing_errors: {hard_fail_on_parsing_errors}')

        if self.parsing_errors and hard_fail_on_parsing_errors:
            logging.debug('hard_fail_on_parsing_errors is True and there were parsing errors - returning 1')
            return 1

        if not self.failed_checks:
            logging.debug('No failed checks in this report - returning 0')
            return 0

        # we will have two different sets of logic in this method, determined by this variable.
        # if we are using enforcement rules, then there are two different sets of thresholds that apply for licenses and vulnerabilities
        # and we have to handle that throughout while processing the report
        # if we are not using enforcement rules, then we can combine licenses and vulnerabilities like normal and same as all other report types
        # this determination is made in runner_registry.get_fail_thresholds
        has_split_enforcement = CodeCategoryType.LICENSES in exit_code_thresholds

        hard_fail_threshold: Optional[Severity | Dict[str, Severity]]
        soft_fail: Optional[bool | Dict[str, bool]]

        if has_split_enforcement:
            sca_thresholds = cast(_ScaExitCodeThresholds, exit_code_thresholds)
            # these three are the same even in split enforcement rules
            generic_thresholds = cast(_ExitCodeThresholds, next(iter(sca_thresholds.values())))
            soft_fail_on_checks = generic_thresholds['soft_fail_checks']
            soft_fail_threshold = generic_thresholds['soft_fail_threshold']
            hard_fail_on_checks = generic_thresholds['hard_fail_checks']

            # these two can be different for licenses / vulnerabilities
            hard_fail_threshold = {category: thresholds['hard_fail_threshold'] for category, thresholds in sca_thresholds.items()}  # type:ignore[index] # thinks it's an object, can't possibly be more clear
            soft_fail = {category: thresholds['soft_fail'] for category, thresholds in sca_thresholds.items()}  # type:ignore[index] # thinks it's an object

            failed_checks_by_category = {
                CodeCategoryType.LICENSES: [fc for fc in self.failed_checks if '_LIC_' in fc.check_id],
                CodeCategoryType.VULNERABILITIES: [fc for fc in self.failed_checks if '_VUL_' in fc.check_id]
            }

            has_soft_fail_values = soft_fail_on_checks or soft_fail_threshold

            if all(
                not failed_checks_by_category[cast(CodeCategoryType, c)] or (
                    not has_soft_fail_values and not (hard_fail_threshold[c] or hard_fail_on_checks) and soft_fail[c]
                )
                for c in sca_thresholds.keys()
            ):
                logging.debug(
                    'No failed checks, or soft_fail is True and soft_fail_on and hard_fail_on are empty for all SCA types - returning 0')
                return 0

            if any(
                not has_soft_fail_values and not (hard_fail_threshold[c] or hard_fail_on_checks) and failed_checks_by_category[cast(CodeCategoryType, c)]
                for c in sca_thresholds.keys()
            ):
                logging.debug('There are failed checks and all soft/hard fail args are empty for one or more SCA reports - returning 1')
                return 1
        else:
            non_sca_thresholds = cast(_ExitCodeThresholds, exit_code_thresholds)
            soft_fail_on_checks = non_sca_thresholds['soft_fail_checks']
            soft_fail_threshold = non_sca_thresholds['soft_fail_threshold']
            hard_fail_on_checks = non_sca_thresholds['hard_fail_checks']
            hard_fail_threshold = non_sca_thresholds['hard_fail_threshold']
            soft_fail = non_sca_thresholds['soft_fail']

            has_soft_fail_values = soft_fail_on_checks or soft_fail_threshold
            has_hard_fail_values = hard_fail_threshold or hard_fail_on_checks

            if not has_soft_fail_values and not has_hard_fail_values and soft_fail:
                logging.debug('Soft_fail is True and soft_fail_on and hard_fail_on are empty - returning 0')
                return 0
            elif not has_soft_fail_values and not has_hard_fail_values:
                logging.debug('There are failed checks and all soft/hard fail args are empty - returning 1')
                return 1

        for failed_check in self.failed_checks:
            check_id = failed_check.check_id
            bc_check_id = failed_check.bc_check_id
            severity = failed_check.severity
            secret_validation_status = failed_check.validation_status if hasattr(failed_check, 'validation_status') else ''

            hf_threshold: Severity
            sf: bool

            if has_split_enforcement:
                category = CodeCategoryType.LICENSES if '_LIC_' in check_id else CodeCategoryType.VULNERABILITIES
                hard_fail_threshold = cast(Dict[str, Severity], hard_fail_threshold)
                hf_threshold = hard_fail_threshold[category]
                soft_fail = cast(Dict[str, bool], soft_fail)
                sf = soft_fail[category]
            else:
                hard_fail_threshold = cast(Severity, hard_fail_threshold)
                hf_threshold = hard_fail_threshold
                soft_fail = cast(bool, soft_fail)
                sf = soft_fail

            soft_fail_severity = severity and soft_fail_threshold and severity.level <= soft_fail_threshold.level
            hard_fail_severity = severity and hf_threshold and severity.level >= hf_threshold.level
            explicit_soft_fail = RunnerFilter.check_matches(check_id, bc_check_id, soft_fail_on_checks)
            explicit_hard_fail = RunnerFilter.check_matches(check_id, bc_check_id, hard_fail_on_checks)
            explicit_secrets_soft_fail = RunnerFilter.secret_validation_status_matches(secret_validation_status, soft_fail_on_checks)
            explicit_secrets_hard_fail = RunnerFilter.secret_validation_status_matches(secret_validation_status, hard_fail_on_checks)
            implicit_soft_fail = not explicit_hard_fail and not explicit_secrets_hard_fail and not soft_fail_on_checks and not soft_fail_threshold
            implicit_hard_fail = not explicit_soft_fail and not soft_fail_severity and not explicit_secrets_soft_fail

            if explicit_hard_fail or \
                    (hard_fail_severity and not explicit_soft_fail) or \
                    (implicit_hard_fail and not implicit_soft_fail and not sf):
                logging.debug(f'Check {check_id} (BC ID: {bc_check_id}, severity: {severity.level if severity else None} triggered hard fail - returning 1')
                return 1

        logging.debug('No failed check triggered hard fail - returning 0')
        return 0

    def is_empty(self, full: bool = False) -> bool:
        checks_count = (
            len(self.passed_checks)
            + len(self.failed_checks)
            + len(self.skipped_checks)
            + len(self.parsing_errors)
        )

        if full:
            checks_count += len(self.resources) + len(self.extra_resources) + len(self.image_cached_results)

        return checks_count == 0

    def print_console(
            self,
            is_quiet: bool = False,
            is_compact: bool = False,
            created_baseline_path: str | None = None,
            baseline: Baseline | None = None,
            use_bc_ids: bool = False,
            summary_position: str = 'top',
            openai_api_key: str | None = None,
    ) -> str:
        summary = self.get_summary()
        output_data = colored(f"{self.check_type} scan results:\n", "blue")
        if self.parsing_errors:
            message = "\nPassed checks: {}, Failed checks: {}, Skipped checks: {}, Parsing errors: {}\n\n".format(
                summary["passed"],
                summary["failed"],
                summary["skipped"],
                summary["parsing_errors"],
            )
        else:
            if self.check_type == CheckType.SCA_PACKAGE:
                message = f"\nFailed checks: {summary['failed']}, Skipped checks: {summary['skipped']}\n\n"
            else:
                message = f"\nPassed checks: {summary['passed']}, Failed checks: {summary['failed']}, Skipped checks: {summary['skipped']}\n\n"
        if summary_position == 'top':
            output_data += colored(message, "cyan")
        # output for vulnerabilities is different
        if self.check_type in (CheckType.SCA_PACKAGE, CheckType.SCA_IMAGE):
            if self.failed_checks or self.skipped_checks:
                create_cli_output = create_sca_package_cli_output_v2 if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 else create_sca_package_cli_output_v1
                output_data += create_cli_output(self.check_type == CheckType.SCA_PACKAGE, self.failed_checks,
                                                 self.skipped_checks)

        elif self.check_type == CheckType.POLICY_3D:
            if self.failed_checks or self.skipped_checks:
                output_data += create_3d_policy_cli_output(self.failed_checks, self.skipped_checks)  # type:ignore[arg-type]

        else:
            if not is_quiet:
                for record in self.passed_checks:
                    output_data += record.to_string(compact=is_compact, use_bc_ids=use_bc_ids)
            if self.failed_checks:
                OpenAi(api_key=openai_api_key).enhance_records(runner_type=self.check_type, records=self.failed_checks)
            for record in self.failed_checks:
                output_data += record.to_string(compact=is_compact, use_bc_ids=use_bc_ids)
            if not is_quiet:
                for record in self.skipped_checks:
                    output_data += record.to_string(compact=is_compact, use_bc_ids=use_bc_ids)

        if not is_quiet:
            for file in self.parsing_errors:
                output_data += colored(f"Error parsing file {file}ֿ\n", "red")

        if created_baseline_path:
            output_data += colored(
                f"Created a checkov baseline file at {created_baseline_path}",
                "blue",
            )
        if baseline:
            output_data += colored(
                f"Baseline analysis report using {baseline.path} - only new failed checks with respect to the baseline are reported",
                "blue",
            )
        if summary_position == 'bottom':
            output_data += colored(message, "cyan")
        return output_data

    @staticmethod
    def _print_parsing_error_console(file: str) -> None:
        print(colored(f"Error parsing file {file}", "red"))

    @staticmethod
    def get_junit_xml_string(ts: list[TestSuite]) -> str:
        return to_xml_report_string(ts)

    def print_failed_github_md(self, use_bc_ids: bool = False) -> str:
        result = []
        for record in self.failed_checks:
            result.append(
                [
                    record.get_output_id(use_bc_ids),
                    record.file_path,
                    record.resource,
                    record.check_name,
                    record.guideline,
                ]
            )
        if result:
            table = tabulate(
                result,
                headers=["check_id", "file", "resource", "check_name", "guideline"],
                tablefmt="github",
                showindex=True,
            )
            output_data = f"### {self.check_type} scan results:\n\n{table}\n\n---\n"
            return output_data
        else:
            return "\n\n---\n\n"

    def get_test_suite(self, properties: Optional[Dict[str, Any]] = None, use_bc_ids: bool = False) -> TestSuite:
        """Creates a test suite for the JUnit XML report"""

        test_cases = []

        records = self.passed_checks + self.failed_checks + self.skipped_checks
        for record in records:
            severity = BcSeverities.NONE
            if record.severity:
                severity = record.severity.name

            if self.check_type == CheckType.SCA_PACKAGE:
                if record.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                if not record.vulnerability_details:
                    # this shouldn't normally happen
                    logging.warning(f"Vulnerability check without details {record.file_path}")
                    continue

                check_id = record.vulnerability_details["id"]
                test_name_detail = f"{record.vulnerability_details['package_name']}: {record.vulnerability_details['package_version']}"
                class_name = f"{record.file_path}.{record.vulnerability_details['package_name']}"
            else:
                check_id = record.bc_check_id if use_bc_ids else record.check_id
                test_name_detail = record.check_name
                class_name = f"{record.file_path}.{record.resource}"

            test_name = f"[{severity}][{check_id}] {test_name_detail}"

            test_case = TestCase(name=test_name, file=record.file_path, classname=class_name)
            if record.check_result["result"] == CheckResult.FAILED:
                test_case.add_failure_info(
                    message=record.check_name,
                    output=self._create_test_case_failure_output(record)
                )
            if record.check_result["result"] == CheckResult.SKIPPED:
                if self.check_type == CheckType.SCA_PACKAGE:
                    test_case.add_skipped_info(f"{check_id} skipped for {test_name_detail}")
                else:
                    test_case.add_skipped_info(record.check_result.get("suppress_comment", ""))

            test_cases.append(test_case)

        test_suite = TestSuite(name=f"{self.check_type} scan", test_cases=test_cases, properties=properties)
        return test_suite

    @staticmethod
    def create_test_suite_properties_block(config: argparse.Namespace) -> Dict[str, Any]:
        """Creates a dictionary without 'None' values for the JUnit XML properties block"""

        properties = {k: v for k, v in config.__dict__.items() if v is not None}
        return properties

    def _create_test_case_failure_output(self, record: Record) -> str:
        """Creates the failure output for a JUnit XML test case

        IaC example:
            Resource: azurerm_network_security_rule.fail_rdp
            File: /main.tf: 71-83
            Guideline: https://docs.bridgecrew.io/docs/bc_azr_networking_2

                    71 | resource "azurerm_network_security_rule" "fail_rdp" {
                    72 |   resource_group_name = azurerm_resource_group.example.name
                    73 |   network_security_group_name=azurerm_network_security_group.example_rdp.name
                    74 |   name                       = "fail_security_rule"
                    75 |   direction                  = "Inbound"
                    76 |   access                     = "Allow"
                    77 |   protocol                   = "TCP"
                    78 |   source_port_range          = "*"
                    79 |   destination_port_range     = "3389"
                    80 |   source_address_prefix      = "*"
                    81 |   destination_address_prefix = "*"
                    82 |   priority = 120
                    83 | }

        SCA example:
            Description: Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover.
            Link: https://nvd.nist.gov/vuln/detail/CVE-2019-19844
            Published Date: 2019-12-18T20:15:00+01:00
            Base Score: 9.8
            Vector: CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H
            Risk Factors: ['Attack complexity: low', 'Attack vector: network', 'Critical severity', 'Has fix']

            Resource: requirements.txt.django
            File: /requirements.txt: 0-0

                    0 | django: 1.2
        """

        failure_output = []

        if self.check_type == CheckType.SCA_PACKAGE:
            if record.vulnerability_details:
                lowest_fixed_version = record.vulnerability_details.get('lowest_fixed_version')
                if lowest_fixed_version is not None:
                    fix = lowest_fixed_version
                else:
                    fixlist = record.vulnerability_details.get('fixed_versions')
                    if fixlist is not None:
                        fix = fixlist

                failure_output.extend(
                    [
                        "",
                        f"Description: {record.description}",
                        f"Link: {record.vulnerability_details.get('link')}",
                        f"Published Date: {record.vulnerability_details.get('published_date')}",
                        f"Base Score: {record.vulnerability_details.get('cvss')}",
                        f"Vector: {record.vulnerability_details.get('vector')}",
                        f"Risk Factors: {record.vulnerability_details.get('risk_factors')}",
                        "Fix Details:",
                        f"  Status: {record.vulnerability_details.get('status')}",
                        f"  Fixed Version: {fix}",
                    ]
                )
            else:
                # this shouldn't normally happen
                logging.warning(f"Vulnerability check without details {record.file_path}")

        failure_output.extend(
            [
                "",
                f"Resource: {record.resource}",
            ]
        )

        if record.file_path:
            file_line = f"File: {record.file_path}"
            if record.file_line_range:
                file_line += f": {record.file_line_range[0]}-{record.file_line_range[1]}"
            failure_output.append(file_line)

        if self.check_type != CheckType.SCA_PACKAGE:
            failure_output.append(f"Guideline: {record.guideline}")

        if record.code_block:
            failure_output.append("")
            failure_output.append(record._code_line_string(code_block=record.code_block, colorized=False))

        return "\n".join(failure_output)

    def print_json(self) -> None:
        print(self.get_json())

    @staticmethod
    def enrich_plan_report(
            report: "Report", enriched_resources: Dict[str, Dict[str, Any]]
    ) -> "Report":
        # This enriches reports with the appropriate filepath, line numbers, and codeblock
        for record in report.failed_checks:
            resource_raw_id = Report.get_plan_resource_raw_id(record.resource)
            enriched_resource = enriched_resources.get(resource_raw_id)
            if enriched_resource:
                record.file_path = enriched_resource["scanned_file"]
                record.file_line_range = enriched_resource["entity_lines_range"]
                record.code_block = enriched_resource["entity_code_lines"]
        return report

    @staticmethod
    def handle_skipped_checks(
            report: "Report", enriched_resources: Dict[str, Dict[str, Any]]
    ) -> "Report":
        module_address_len = len("module.")
        skip_records = []
        for record in report.failed_checks:
            resource_raw_id = Report.get_plan_resource_raw_id(record.resource)
            resource_skips = enriched_resources.get(resource_raw_id, {}).get(
                "skipped_checks", []
            )
            for skip in resource_skips:
                if record.check_id in skip["id"]:
                    # Mark for removal and add it as a skipped record. It is not safe to remove
                    # the record from failed_checks immediately because we're iterating over it
                    skip_records.append(record)
                    record.check_result["result"] = CheckResult.SKIPPED
                    record.check_result["suppress_comment"] = skip["suppress_comment"]
                    report.add_record(record)

            if record.resource_address and record.resource_address.startswith("module."):
                module_path = record.resource_address[module_address_len:record.resource_address.index('.', module_address_len + 1)]
                module_enrichments = enriched_resources.get(module_path, {})
                for module_skip in module_enrichments.get("skipped_checks", []):
                    if record.check_id in module_skip["id"]:
                        skip_records.append(record)
                        record.check_result["result"] = CheckResult.SKIPPED
                        record.check_result["suppress_comment"] = module_skip["suppress_comment"]
                        report.add_record(record)

        for record in skip_records:
            if record in report.failed_checks:
                report.failed_checks.remove(record)
        return report

    @staticmethod
    def get_plan_resource_raw_id(resource_id: str) -> str:
        """
        return the resource raw id without the modules and the indexes
        example: from resource_id='module.module_name.type.name[1]' return 'type.name'
        """
        resource_raw_id = ".".join(resource_id.split(".")[-2:])
        if '[' in resource_raw_id:
            resource_raw_id = resource_raw_id[:resource_raw_id.index('[')]
        return resource_raw_id

    @classmethod
    def from_reduced_json(cls, json_report: dict[str, Any], check_type: str) -> Report:
        report = Report(check_type)
        report.image_cached_results = json_report['image_cached_results']

        all_json_records = json_report["checks"]["passed_checks"] + \
            json_report["checks"]["failed_checks"] + \
            json_report["checks"]["skipped_checks"]

        for json_record in all_json_records:
            report.add_record(
                Record.from_reduced_json(json_record)
            )

        return report


def merge_reports(base_report: Report, report_to_merge: Report) -> None:
    base_report.passed_checks.extend(report_to_merge.passed_checks)
    base_report.failed_checks.extend(report_to_merge.failed_checks)
    base_report.skipped_checks.extend(report_to_merge.skipped_checks)
    base_report.parsing_errors.extend(report_to_merge.parsing_errors)
    base_report.image_cached_results.extend(report_to_merge.image_cached_results)
    base_report.resources.update(report_to_merge.resources)
    base_report.extra_resources.update(report_to_merge.extra_resources)


def remove_duplicate_results(report: Report) -> Report:
    def dedupe_records(origin_records: list[Record]) -> list[Record]:
        unique_records: Dict[str, Record] = {}
        for record in origin_records:
            record_hash = record.get_unique_string()
            unique_records[record_hash] = record

        return list(unique_records.values())

    report.passed_checks = dedupe_records(report.passed_checks)
    report.failed_checks = dedupe_records(report.failed_checks)
    return report



from typing import Dict, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class WAFACLCVE202144228(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure WAF prevents message lookup in Log4j2. See CVE-2021-44228 aka log4jshell"
        id = "CKV_AWS_192"
        supported_resources = ["aws_wafv2_web_acl"]
        categories = [CheckCategories.APPLICATION_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["rule"]
        rules = conf.get("rule") or []
        for idx_rule, rule in enumerate(force_list(rules)):
            self.evaluated_keys = [f"rule/[{idx_rule}]/statement"]
            statement = rule.get("statement")
            if statement:
                self.evaluated_keys = [f"rule/[{idx_rule}]/statement/[0]/managed_rule_group_statement"]
                if not isinstance(statement, list):
                    return CheckResult.UNKNOWN
                managed_group = statement[0].get("managed_rule_group_statement")
                if managed_group:
                    self.evaluated_keys = [f"rule/[{idx_rule}]/statement/[0]/managed_rule_group_statement/[0]/name"]
                    if managed_group[0] and managed_group[0].get("name") == ["AWSManagedRulesKnownBadInputsRuleSet"]:
                        self.evaluated_keys.append(
                            f"rule/[{idx_rule}]/statement/[0]/managed_rule_group_statement/[0]/excluded_rule"
                        )
                        excluded_rules = managed_group[0].get("excluded_rule") or []
                        # rule 'Log4JRCE' should not be set to count
                        for idx_excluded_rule, excluded_rule in enumerate(force_list(excluded_rules)):
                            if excluded_rule and excluded_rule.get("name") == ["Log4JRCE"]:
                                self.evaluated_keys = [
                                    f"rule/[{idx_rule}]/statement/[0]/managed_rule_group_statement/[0]/name",
                                    f"rule/[{idx_rule}]/statement/[0]/managed_rule_group_statement/[0]/excluded_rule/[{idx_excluded_rule}]/name",
                                ]
                                return CheckResult.FAILED

                        self.evaluated_keys.append(
                            f"rule/[{idx_rule}]/override_action/[0]/none"
                        )
                        override_action = rule.get("override_action")
                        # check for group override
                        override_action_none = override_action[0].get("none")
                        # Terraform plan includes both keys, but one is a dict and the not chosen one a list
                        if not override_action_none or not isinstance(override_action_none[0], dict):
                            return CheckResult.FAILED

                        return CheckResult.PASSED

        return CheckResult.FAILED


check = WAFACLCVE202144228()



from __future__ import annotations

from typing import Any
from abc import abstractmethod

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.json_utils import get_jsonpath_from_evaluated_key
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.branch_protection import schema as branch_security_schema
from checkov.github.schemas.no_branch_protection import schema as no_branch_security_schema
from checkov.json_doc.enums import BlockType

MESSAGE_BRANCH_NOT_PROTECTED = 'Branch not protected'


class BranchSecurity(BaseGithubCheck):
    def __init__(self, id: str, name: str) -> None:
        categories = (CheckCategories.SUPPLY_CHAIN,)
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=("*",),
            block_type=BlockType.DOCUMENT,
        )

    def scan_entity_conf(  # type:ignore[override]
            self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        if branch_security_schema.validate(conf):
            evaluated_key = self.get_evaluated_keys()[0]
            jsonpath_expression = get_jsonpath_from_evaluated_key(evaluated_key)
            matches = jsonpath_expression.find(conf)
            if matches and all(isinstance(match.value, dict) or match.value == self.get_expected_value() for match in matches):
                return CheckResult.PASSED
            else:
                return CheckResult.FAILED
        if no_branch_security_schema.validate(conf):
            message = conf.get('message', '')
            if message == MESSAGE_BRANCH_NOT_PROTECTED:
                return CheckResult.FAILED
        return CheckResult.UNKNOWN

    def get_expected_value(self) -> str | bool:
        return True

    @abstractmethod
    def get_evaluated_keys(self) -> list[str]:
        pass



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class GoogleComputeBootDiskEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure VM disks for critical VMs are encrypted with Customer Supplied Encryption Keys (CSEK)"
        id = "CKV_GCP_38"
        supported_resources = ['google_compute_instance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        self.evaluated_keys = ['boot_disk', 'boot_disk/[0]/disk_encryption_key_raw', 'boot_disk/[0]/kms_key_self_link']
        if 'boot_disk' in conf.keys() and (('disk_encryption_key_raw' in conf['boot_disk'][0] and
                                            conf['boot_disk'][0]['disk_encryption_key_raw'][0] is not None) or
                                           ('kms_key_self_link' in conf['boot_disk'][0] and
                                            conf['boot_disk'][0]['kms_key_self_link'][0] is not None)):
            return CheckResult.PASSED
        return CheckResult.FAILED


check = GoogleComputeBootDiskEncryption()



import dpath
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSRbacEnabled(BaseResourceCheck):
    def __init__(self):
        name = "Ensure RBAC is enabled on AKS clusters"
        id = "CKV_AZURE_5"
        supported_resources = ["azurerm_kubernetes_cluster"]
        categories = [CheckCategories.KUBERNETES]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf):
        self.evaluated_keys = [
            "role_based_access_control/[0]/enabled",  # azurerm < 2.99.0
            "role_based_access_control_enabled",  # azurerm >= 2.99.0
        ]

        for key in self.evaluated_keys:
            if dpath.search(conf, key):
                return CheckResult.PASSED if dpath.get(conf, key)[0] else CheckResult.FAILED

        return CheckResult.PASSED


check = AKSRbacEnabled()



from __future__ import annotations

from typing import Dict, Any, TYPE_CHECKING

from checkov.common.graph.graph_builder import CustomAttributes

if TYPE_CHECKING:
    from igraph import Graph


def serialize_to_json(igraph: Graph, absolute_root_folder: str = '') -> Dict[str, Any]:
    nodes = []
    for i, vertex in enumerate(igraph.vs):
        attr = {k: v for k, v in vertex.attributes()['attr'].items() if v is not None}
        node = {'attr': attr, "id": vertex.attributes().get('block_index', i),
                "name": vertex.attributes().get('name', attr[CustomAttributes.HASH]),
                CustomAttributes.BLOCK_TYPE: vertex[CustomAttributes.BLOCK_TYPE],
                CustomAttributes.RESOURCE_TYPE: attr[CustomAttributes.RESOURCE_TYPE] if CustomAttributes.RESOURCE_TYPE in attr else None}
        nodes.append(node)

    links = [
        {
            "label": edge["label"],
            "source": edge.source,
            "target": edge.target
        }
        for edge in igraph.es
    ]

    graph = {"graph_type": "igraph", "absolute_root_folder": absolute_root_folder, "nodes": nodes, "links": links}

    return graph
